/*===========================================================================
 *
 * File:	Dl_blowf.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	April 29, 2003
 *
 * Description
 *	Simple BlowFish encryption implementation as derived from the C 
 *	source as written by Paul Kocher (pck@netcom.com, 1997).
 * 
 *========================================================================*/

	/* Include Files */
#include "dl_blowf.h"
#include "dl_time.h"
#include "dl_str.h"
#include "limits.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("dl_BlowF.cpp");

	/* The S/P data arrays */
  #include "dl_BFDat.h"
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CBlowFishAlg Constructor
 *
 *=========================================================================*/
CBlowFishAlg::CBlowFishAlg (const TCHAR* pKeyName) {
  //DEFINE_FUNCTION("CBlowFishAlg::CBlowFishAlg()");

	/* Default pad unaligned data with spaces */
  m_PadByte = ' ';

  Initialize(pKeyName);
 }
/*===========================================================================
 *		End of Class CBlowFishAlg Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CBlowFishAlg Method - void Destroy (void);
 *
 *=========================================================================*/
void CBlowFishAlg::Destroy (void) {
  //DEFINE_FUNCTION("CBlowFishAlg::Destroy()");
 }
/*===========================================================================
 *		End of Class Method CBlowFishAlg::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CBlowFishAlg Method - dword Convert (Val);
 *
 * Internal protected class method that converts and returns the given value.
 *
 *=========================================================================*/
dword CBlowFishAlg::Convert (const dword Val) {
   /*word  Byte1;
   word  Byte2;
   word  Byte3;
   word  Byte4;
   dword Result;
 
   Byte1 = (Val >>  0) & 0x00FF;
   Byte2 = (Val >>  8) & 0x00FF;
   Byte3 = (Val >> 16) & 0x00FF;
   Byte4 = (Val >> 24) & 0x00FF;

   Result  = m_SData[0][Byte4] + m_SData[1][Byte3];
   Result ^= m_SData[2][Byte2];
   Result += m_SData[3][Byte1];

   return (Result); //*/

	/* One line version of the above */
  return (((m_SData[0][(Val >> 24) & 0x00FF]  + m_SData[1][(Val >> 16) & 0x00FF]) ^ 
            m_SData[2][(Val >>  8) & 0x00FF]) + m_SData[3][(Val >>  0) & 0x00FF]);
 }
/*===========================================================================
 *		End of Class Method CBlowFishAlg::Convert()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CBlowFishAlg Method - void DecryptBlock (XLeft, XRight);
 *
 * Protected class method that takes the input encrypted data and returns
 * the decrypted information in-place.
 *
 *=========================================================================*/
void CBlowFishAlg::DecryptBlock (dword& XLeft, dword& XRight) {
  dword SwapTemp;
  dword Xl = XLeft;
  dword Xr = XRight;
  short Index;

  for (Index = BFALG_PDATA_LOOPSIZE+1; Index > 1; Index--) {
    Xl = Xl ^ m_PData[Index];
    Xr = Convert(Xl) ^ Xr;

    SwapTemp = Xl;
    Xl       = Xr;
    Xr       = SwapTemp;
   }

  SwapTemp = Xl;
  Xl       = Xr;
  Xr       = SwapTemp;

  XRight = Xr ^ m_PData[1];
  XLeft  = Xl ^ m_PData[0];
 }
/*===========================================================================
 *		End of Class Method CBlowFishAlg::DecryptBlock()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CBlowFishAlg Method - void DecryptBuffer (pBuffer, BufferSize);
 *
 * Attempt to decrypt the given data buffer in place. The buffer size should
 * be a multiple of 8. If it isn't, the last 0-7 bytes are not decoded.
 * The buffer size is updated to match the amount of buffer data decrypted.
 *
 *=========================================================================*/
void CBlowFishAlg::DecryptBuffer (byte* pBuffer, int &BufferSize) {
  DEFINE_FUNCTION("CBlowFishAlg::DecryptBuffer()");
  int   BufferPos;
  dword Value1;
  dword Value2;

	/* Ensure valid input */
  ASSERT(pBuffer != NULL && BufferSize > 0);
  BufferPos = 0;

	/* Convert as much of the input buffer as possible */
  while (BufferPos + BFALG_MIN_DATASIZE <= BufferSize) {
    Value1 = *(dword *)(pBuffer + BufferPos);
    Value2 = *(dword *)(pBuffer + BufferPos + 4);

    DecryptBlock(Value1, Value2);

    *(dword *)(pBuffer + BufferPos)     = Value1;
    *(dword *)(pBuffer + BufferPos + 4) = Value2;

    BufferPos += BFALG_MIN_DATASIZE;
   }

	/* Store the amount of bytes converted successfully */
  BufferSize = BufferPos;
 }
/*===========================================================================
 *		End of Class Method CBlowFishAlg::DecryptBuffer()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CBlowFishAlg Method - void EncryptBlock (XLeft, XRight);
 *
 * Protected class method that takes the input data and returns the 
 * encrypted information in-place.
 *
 *=========================================================================*/
void CBlowFishAlg::EncryptBlock (dword& XLeft, dword& XRight) {
  dword SwapTemp;
  dword Xl = XLeft;
  dword Xr = XRight;
  short Index;

  for (Index = 0; Index < BFALG_PDATA_LOOPSIZE; Index++) {
    Xl = Xl ^ m_PData[Index];
    Xr = Convert(Xl) ^ Xr;

    SwapTemp = Xl;
    Xl       = Xr;
    Xr       = SwapTemp;
   }

  SwapTemp = Xl;
  Xl       = Xr;
  Xr       = SwapTemp;

  XRight = Xr ^ m_PData[BFALG_PDATA_LOOPSIZE];
  XLeft  = Xl ^ m_PData[BFALG_PDATA_LOOPSIZE + 1];
 }
/*===========================================================================
 *		End of Class Method CBlowFishAlg::EncryptBlock()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CBlowFishAlg Method - void EncryptBuffer (pBuffer, BufferSize);
 *
 * Transforms an arbitrarily long data buffer in place. If the input buffer
 * size is not a multiple of 8, the output buffer will be padded with the 
 * current pad byte character. Ensure the buffer is at least 8 bytes too large
 * or simiarily ensure it is large enough.
 *
 * For NULL terminated string encryption the input size should include the 
 * terminating NULL byte.
 *
 *=========================================================================*/
void CBlowFishAlg::EncryptBuffer (byte* pBuffer, int& BufferSize) {
  DEFINE_FUNCTION("CBlowFishAlg::DecryptBuffer()");
  int   BufferPos;
  dword Value1;
  dword Value2;

	/* Ensure valid input */
  ASSERT(pBuffer != NULL && BufferSize > 0);

	/* Pad the input data as required */
  if ((BufferSize % BFALG_MIN_DATASIZE) != 0) {
    BufferPos  = BufferSize;
    BufferSize = BufferSize + BFALG_MIN_DATASIZE - (BufferSize % BFALG_MIN_DATASIZE);

    for (; BufferPos < BufferSize; BufferPos++) {
      pBuffer[BufferPos] = m_PadByte;
     }
   }

  BufferPos = 0;

	/* Convert the entire input buffer */
  while (BufferPos <= BufferSize) {
    Value1 = *(dword *)(pBuffer + BufferPos);
    Value2 = *(dword *)(pBuffer + BufferPos + 4);
 
    EncryptBlock(Value1, Value2);

    *(dword *)(pBuffer + BufferPos)     = Value1;
    *(dword *)(pBuffer + BufferPos + 4) = Value2;

    BufferPos += BFALG_MIN_DATASIZE;
   }

 }
/*===========================================================================
 *		End of Class Method CBlowFishAlg::EncryptBuffer()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CBlowFishAlg Method - void Initialize (pKeyName);
 *
 * Initializes the BlowFish algorithm with the given key string.
 *
 *=========================================================================*/
void CBlowFishAlg::Initialize (const TCHAR* pKeyName) {
  DEFINE_FUNCTION("CBlowFishAlg::Initialize()");
  dword Data;
  dword DataL;
  dword DataR;
  int   DataIndex;
  int   Index;
  int	KeyIndex;
  int   KeyLength;

  ASSERT(pKeyName != NULL);
  KeyLength = TSTRLEN(pKeyName);

	/* Copy the initial data tables */
  memcpy(m_PData, g_BFPData, sizeof(g_BFPData));
  memcpy(m_SData, g_BFSData, sizeof(g_BFSData));
  KeyIndex = 0;

  for (DataIndex = 0; DataIndex < BFALG_PDATA_SIZE; DataIndex++) {
    Data = 0;

    for (Index = 0; Index < BFALG_SDATA_SIZE1; Index++) {
      Data = (Data << 8) | pKeyName[KeyIndex];
      KeyIndex++;
      if (KeyIndex >= KeyLength || KeyLength >= BFALG_KEY_MAXSIZE) KeyIndex = 0;
     }

    m_PData[DataIndex] = m_PData[DataIndex] ^ Data;
   }

  DataL = 0;
  DataR = 0;

  for (DataIndex = 0; DataIndex < BFALG_PDATA_SIZE; DataIndex += 2) {
    EncryptBlock(DataL, DataR);
    m_PData[DataIndex]   = DataL;
    m_PData[DataIndex+1] = DataR;
   }

  for (DataIndex = 0; DataIndex < BFALG_SDATA_SIZE1; DataIndex++) {
    for (Index = 0; Index < BFALG_SDATA_SIZE2; Index += 2) {
      EncryptBlock(DataL, DataR);
      m_SData[DataIndex][Index]     = DataL;
      m_SData[DataIndex][Index + 1] = DataR;
     }
   }

 }
/*===========================================================================
 *		End of Class Method CBlowFishAlg::Initialize()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CBlowFishAlg Method - bool StressTest (NumLoops);
 *
 * Performs a more thorough test by encrypting and decrypting random numbers
 * for the given number of iterations. Returns false on any error.
 *
 *=========================================================================*/
bool CBlowFishAlg::StressTest (const long NumLoops) {
  //DEFINE_FUNCTION("CBlowFishAlg::StressTest()");
  double StartTime;
  double EndTime;
  long   LoopIndex;
  dword  Value1;
  dword  Value2;
  dword  Result1;
  dword  Result2;

	/* Randomize the timer */
  srand(time(NULL));
  StartTime = GetHiClockTime();

	/* Perform the desired number of iterations */
  for (LoopIndex = 0; LoopIndex < NumLoops; LoopIndex++) {

		/* Generate the two random numbers */
    Result1 = Value1 = (dword) (rand() * ULONG_MAX);
    Result2 = Value2 = (dword) (rand() * ULONG_MAX);

		/* Encrypt and decrypt */
    EncryptBlock(Result1, Result2);
    DecryptBlock(Result1, Result2);

		/* Check results */
    if (Result1 != Value1 || Result2 != Value2) {
      ErrorHandler.AddError(ERR_CUSTOM, "%ld) Error with values 0x%08lX, 0x%08lX", LoopIndex+1, Value1, Value2);
      return (false);
     }
   }

	/* Output statistics */
  EndTime = GetHiClockTime();
  SystemLog.Printf("Successfully performed %ld checks in %g secs (%g ms/check).", LoopIndex, EndTime-StartTime, (EndTime-StartTime)/(LoopIndex+1.0)*1000.0);

  return (true);
 }
/*===========================================================================
 *		End of Class Method CBlowFishAlg::StressTest()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CBlowFishAlg Method - bool Test (void);
 *
 * Performs a quick test of the algorithm, returning false on any error.
 * This method re-initializes the current key.
 *
 *=========================================================================*/
bool CBlowFishAlg::Test (void) {
  //DEFINE_FUNCTION("CBlowFishAlg::Test()");
  dword		LValue = 1;
  dword		RValue = 2;
  const TCHAR*	pTestString = "Test string";
  TCHAR		Buffer[256];
  int		BufferSize;

	/* Initialize the algorithm with the test key string */
  Initialize ("TESTKEY");

	/* Test the encryption method */
  EncryptBlock(LValue, RValue);

  if (LValue != 0xDF333FD2L || RValue != 0x30A71BB4L) {
    ErrorHandler.AddError(ERR_CUSTOM, "CBlowFishAlg::EncryptBlock() method failed!");
    return (false);
   }

	/* Test the decryption method */
  DecryptBlock(LValue, RValue);

  if (LValue != 1 || RValue != 2) {
    ErrorHandler.AddError(ERR_CUSTOM, "CBlowFishAlg::DecryptBlock() method failed!");
    return (false);
   }

	/* Test the buffer encryption */
  strnncpy(Buffer, pTestString, 255);
  BufferSize = strlen(Buffer) + 1; 

  EncryptBuffer((byte *)Buffer, BufferSize);
  DecryptBuffer((byte *)Buffer, BufferSize);

  if (strcmp(Buffer, pTestString) != 0) {
    ErrorHandler.AddError(ERR_CUSTOM, "Error with CBlowFishAlg buffer methods!");
    return (false);
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CBlowFishAlg::Test()
 *=========================================================================*/
