/*===========================================================================
 *
 * File:	Dfwoods.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	May 30, 2003
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "dfwoods.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("DfWoods.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Constructor
 *
 *=========================================================================*/
CDfWoodsFile::CDfWoodsFile () {
  //DEFINE_FUNCTION("CDfWoodsFile::CDfWoodsFile()");
  memset(&m_Header, 0, sizeof(dfwoodsheader_t));
  m_HasHeader = false;
  m_pSmallMap = NULL;
  m_SmallMapSize = 0;
 }
/*===========================================================================
 *		End of Class CDfWoodsFile Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - void Destroy (void);
 *
 *=========================================================================*/
void CDfWoodsFile::Destroy (void) {
  DEFINE_FUNCTION("CDfWoodsFile::Destroy()");

  memset(&m_Header, 0, sizeof(dfwoodsheader_t));
  m_HasHeader = false;
  DestroyArrayPointer(m_pSmallMap);
  m_SmallMapSize = 0;

  CGenFile::Destroy();
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool AllocateSmallMap (void);
 *
 * Protected class method to allocate the small map data array. Returns
 * false on any error.
 *
 *=========================================================================*/
bool CDfWoodsFile::AllocateSmallMap (void) {
  DEFINE_FUNCTION("CDfWoodsFile::AllocateSmallMap()");
  ulong MapSize;
  
	/* Delete the current map, if any */
  DestroyPointer(m_pSmallMap);

	/* Compute the map size */
  MapSize = (ulong)m_Header.ImageHeight * (ulong)m_Header.ImageWidth;

  if (MapSize >= (ulong)UINT_MAX) {
    ErrorHandler.AddError(ERR_MEM, _T("Cannot allocate %ul bytes!"), MapSize);
    return (false);
   }

	/* Allocate the pointer */
  CreateArrayPointer(m_pSmallMap, byte, MapSize);
  m_SmallMapSize = (size_t)MapSize;

  return (true);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::AllocateSmallMap()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool ExportSmallMap (pFilename);
 *
 * Exports the small map data to the given file. Returns false on any error.
 *
 *=========================================================================*/
bool CDfWoodsFile::ExportSmallMap (const TCHAR* pFilename) {
  CGenFile OutputFile;
  byte*    pMapData;
  bool	   Result;

	/* Get the small map data, loading if required */
  pMapData = GetSmallMap();
  if (pMapData == NULL) return (false);

	/* Attempt to open the output file */
  Result = OutputFile.Open(pFilename, "wb");
  if (!Result) return (false);

	/* Write the map data */
  Result = OutputFile.Write((char *) pMapData, m_SmallMapSize);
  return (Result);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::ExportSmallMap()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool ExportLargeMap (pFilename);
 *
 * Exports the large map data to the given file. Returns false on any error.
 *
 *=========================================================================*/
bool CDfWoodsFile::ExportLargeMap (const TCHAR* pFilename) {
  dfwoodspixel_t PixelData;
  CGenFile	 OutputFile;
  bool		 Result;
  int		 XPixel;
  int		 YPixel;
  long		 Offset;

	/* Ensure the header is loaded */
  if (!m_HasHeader && !ReadHeader()) return (false);

	/* Attempt to open the output file */
  Result = OutputFile.Open(pFilename, "wb");
  if (!Result) return (false);

	/* Output all the pixel data */
  for (YPixel = 0; YPixel < m_Header.ImageHeight; YPixel++) {
    for (XPixel = 0; XPixel < m_Header.ImageWidth; XPixel++) {
      Offset = XPixel*5 + YPixel*m_Header.ImageWidth*25;

		/* Input the pixel data */
      Result = ReadPixel(PixelData, XPixel, YPixel);
      if (!Result) return (false);

		/* Output the pixel data */
      Result  = OutputFile.Seek(Offset, SEEK_SET);
      Result &= OutputFile.Write((char *)PixelData.MapData[0], 5);
      Offset  = m_Header.ImageWidth*5 - 5;
      Result &= OutputFile.Seek(Offset, SEEK_CUR);
      Result &= OutputFile.Write((char *)PixelData.MapData[1], 5);
      Result &= OutputFile.Seek(Offset, SEEK_CUR);
      Result &= OutputFile.Write((char *)PixelData.MapData[2], 5);
      Result &= OutputFile.Seek(Offset, SEEK_CUR);
      Result &= OutputFile.Write((char *)PixelData.MapData[3], 5);
      Result &= OutputFile.Seek(Offset, SEEK_CUR);
      Result &= OutputFile.Write((char *)PixelData.MapData[4], 5);
      if (!Result) return (false);
     }
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::ExportLargeMap()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool ExportClimate (pFilename);
 *
 * Exports the climate map data to the given file. Returns false on any error.
 *
 *=========================================================================*/
bool CDfWoodsFile::ExportClimate (const TCHAR* pFilename) {
  dfwoodspixel_t PixelData;
  CGenFile	 OutputFile;
  bool		 Result;
  int		 XPixel;
  int		 YPixel;

	/* Ensure the header is loaded */
  if (!m_HasHeader && !ReadHeader()) return (false);

	/* Attempt to open the output file */
  Result = OutputFile.Open(pFilename, "wb");
  if (!Result) return (false);

	/* Output all the pixel data */
  for (YPixel = 0; YPixel < m_Header.ImageHeight; YPixel++) {
    for (XPixel = 0; XPixel < m_Header.ImageWidth; XPixel++) {

		/* Input the pixel data */
      Result = ReadPixel(PixelData, XPixel, YPixel);
      if (!Result) return (false);

		/* Output the pixel data */
      Result = OutputFile.WriteChar((char)PixelData.ClimateType);
      if (!Result) return (false);
     }
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::ExportClimate()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool ExportType (pFilename);
 *
 * Exports the land type map data to the given file. Returns false on any error.
 *
 *=========================================================================*/
bool CDfWoodsFile::ExportType (const TCHAR* pFilename) {
  dfwoodspixel_t PixelData;
  CGenFile	 OutputFile;
  bool		 Result;
  int		 XPixel;
  int		 YPixel;

	/* Ensure the header is loaded */
  if (!m_HasHeader && !ReadHeader()) return (false);

	/* Attempt to open the output file */
  Result = OutputFile.Open(pFilename, "wb");
  if (!Result) return (false);

	/* Output all the pixel data */
  for (YPixel = 0; YPixel < m_Header.ImageHeight; YPixel++) {
    for (XPixel = 0; XPixel < m_Header.ImageWidth; XPixel++) {

		/* Input the pixel data */
      Result = ReadPixel(PixelData, XPixel, YPixel);
      if (!Result) return (false);

		/* Output the pixel data */
      Result = OutputFile.WriteChar((char)PixelData.LandType);
      if (!Result) return (false);
     }
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::ExportType()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool ExportHeight (pFilename);
 *
 * Exports the land height map data to the given file. Returns false on any error.
 *
 *=========================================================================*/
bool CDfWoodsFile::ExportHeight (const TCHAR* pFilename) {
  dfwoodspixel_t PixelData;
  CGenFile	 OutputFile;
  bool		 Result;
  int		 XPixel;
  int		 YPixel;

	/* Ensure the header is loaded */
  if (!m_HasHeader && !ReadHeader()) return (false);

	/* Attempt to open the output file */
  Result = OutputFile.Open(pFilename, "wb");
  if (!Result) return (false);

	/* Output all the pixel data */
  for (YPixel = 0; YPixel < m_Header.ImageHeight; YPixel++) {
    for (XPixel = 0; XPixel < m_Header.ImageWidth; XPixel++) {

		/* Input the pixel data */
      Result = ReadPixel(PixelData, XPixel, YPixel);
      if (!Result) return (false);

		/* Output the pixel data */
      Result = OutputFile.WriteChar((char)PixelData.LandHeight);
      if (!Result) return (false);
     }
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::ExportHeight()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool ExportSeed (pFilename);
 *
 * Exports the seed map data to the given file. Returns false on any error.
 *
 *=========================================================================*/
bool CDfWoodsFile::ExportSeed (const TCHAR* pFilename) {
  dfwoodspixel_t PixelData;
  CGenFile	 OutputFile;
  bool		 Result;
  int		 XPixel;
  int		 YPixel;
  
	/* Ensure the header is loaded */
  if (!m_HasHeader && !ReadHeader()) return (false);

	/* Attempt to open the output file */
  Result = OutputFile.Open(pFilename, "wb");
  if (!Result) return (false);

	/* Output all the pixel data */
  for (YPixel = 0; YPixel < m_Header.ImageHeight; YPixel++) {
    for (XPixel = 0; XPixel < m_Header.ImageWidth; XPixel++) {

		/* Input the pixel data */
      Result = ReadPixel(PixelData, XPixel, YPixel);
      if (!Result) return (false);

		/* Output the pixel data */
      Result = OutputFile.WriteChar((char)PixelData.NumberSeed);
      if (!Result) return (false);
     }
   }

  return (true);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::ExportSeed()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - long GetPixelOffset (XPixel, YPixel);
 *
 * Returns the offset of the start of the given pixel data from the start
 * of the file. Returns -1 on any error. Assumes that the header data has
 * been read.
 *
 *=========================================================================*/
long CDfWoodsFile::GetPixelOffset (const int XPixel, const int YPixel) {
  long Offset;
  bool Result;

	/* Ensure a valid pixel coordinate */
  if (!IsValidPixel(XPixel, YPixel)) return (-1);

	/* Compute the offset to the offset */
  Offset = (XPixel + YPixel*m_Header.ImageWidth)*4;
  if (Offset < 0 || Offset >= m_Header.OffsetSize) return (-1);
  Offset += sizeof(dfwoodsheader_t);

	/* Read in the offset to the data */
  Result = Seek(Offset, SEEK_SET);
  if (!Result) return (-1);
  Result = ReadLong(Offset);
  if (!Result) return (-1);

  return (Offset);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::GetPixelOffset()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - byte* GetSmallMap (void);
 *
 * Returns the small map data array, or NULL on any error. Reads the 
 * header and the data if required.
 *
 *=========================================================================*/
byte* CDfWoodsFile::GetSmallMap (void) {
  //DEFINE_FUNCTION("CDfWoodsFile::GetSmallMap()");
  bool Result;

	/* Just return the data if we've already read it */
  if (m_HasHeader && m_pSmallMap != NULL) return (m_pSmallMap);

	/* Input the header if required */
  if (!m_HasHeader) {
    Result = ReadHeader();
    if (!Result) return (false);
   }

	/* Input the map data */
  Result = ReadSmallMap();
  if (!Result) return (false);

  return (m_pSmallMap);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::GetSmallMap()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool IsValidPixel (X, Y);
 *
 * Returns true if the given pixel coordinate is inside the small map
 * image (top-left is 0,0).
 *
 *=========================================================================*/
bool CDfWoodsFile::IsValidPixel (const int X, const int Y) {
  return (X >= 0 && Y >= 0 && X < m_Header.ImageWidth && Y < m_Header.ImageHeight);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::IsValidPixel()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool ReadHeader (void);
 *
 * Reads the header from the start of the file. Protected class method.
 * Returns false on any error.
 *
 *=========================================================================*/
bool CDfWoodsFile::ReadHeader (void) {
  DEFINE_FUNCTION("CDfWoodsFile::ReadHeader()");
  bool Result;

  m_HasHeader = false;

	/* Attempt to move to the start of the file */
  Result = Seek(0, SEEK_SET);
  if (!Result) return (false);

	/* Return the header data all at once */
  Result = Read((char *)&m_Header, sizeof(dfwoodsheader_t));
  m_HasHeader = Result;
  return (Result);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::ReadHeader()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool ReadPixel (PixelData, XPixel, YPixel);
 *
 * Protected class method which attempts to input the given pixel data.
 * Assumes that the file has been successfully opened and the header read.
 *
 *=========================================================================*/
bool CDfWoodsFile::ReadPixel (dfwoodspixel_t& PixelData, const int XPixel, const int YPixel) {
  long Offset;
  bool Result;

	/* Compute the offset for the pixel data */
  Offset = GetPixelOffset(XPixel, YPixel);
  if (Offset < 0) return (false);

	/* Seek and read in the data */
  Result = Seek(Offset, SEEK_SET);
  if (!Result) return (false);
  Result = Read((char *)&PixelData, sizeof(dfwoodspixel_t));
  return (Result);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::ReadPixel()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - dfwoodspixshrt_t* ReadShortPixelData (void);
 *
 * Allocates and reads a short pixel data array for each pixel in the
 * map data. Returns NULL on any error. Reads the header if required.
 * Assumes that the file has already been opened.
 *
 *=========================================================================*/
dfwoodspixshrt_t* CDfWoodsFile::ReadShortPixelData (void) {
  DEFINE_FUNCTION("CDfWoodsFile::ReadShortPixelData()");
  dfwoodspixshrt_t*	pPixelData;
  dfwoodspixel_t	PixelLongData;
  bool			Result;
  int			XPixel;
  int			YPixel;
  size_t		Index;

	/* Input the header and small map data */
  if (GetSmallMap() == NULL) return (false);
   
	/* Allocate the pixel data array */
  CreateArrayPointer(pPixelData, dfwoodspixshrt_t, m_SmallMapSize);
  XPixel = 0;
  YPixel = 0;

	/* Read in all the pixel data and save it to the array */
  for (Index = 0; Index < m_SmallMapSize; Index++) {

		/* Read the long pixel data */
    Result = ReadPixel(PixelLongData, XPixel, YPixel);

    if (!Result) {
      DestroyPointer(pPixelData);
      return (NULL);
     }

		/* Save the short pixel data */
    pPixelData[Index].Color = m_pSmallMap[Index];
    pPixelData[Index].Flags = PixelLongData.LandHeight;

    switch (PixelLongData.ClimateType) {
      case DFWOOD_TILESET_DESERT:	pPixelData[Index].Flags |= DFWOOD_FLAG_DESERT; break;
      case DFWOOD_TILESET_MOUNTAIN:	pPixelData[Index].Flags |= DFWOOD_FLAG_MOUNTAIN; break;
      case DFWOOD_TILESET_TEMPERATE:	pPixelData[Index].Flags |= DFWOOD_FLAG_TEMPERATE; break;
      case DFWOOD_TILESET_JUNGLE:	pPixelData[Index].Flags |= DFWOOD_FLAG_JUNGLE; break;
     };

		/* Increment the x/y position */
    XPixel++;

    if (XPixel >= m_Header.ImageWidth) {
      XPixel = 0;
      YPixel++;
      if (YPixel >= m_Header.ImageHeight) break;
     }
   }

  return (pPixelData);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::ReadShortPixelData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CDfWoodsFile Method - bool ReadSmallMap (void);
 *
 * Protected class method to read in the small map data. Returns false
 * on any error. The header must have been previously read.
 *
 *=========================================================================*/
bool CDfWoodsFile::ReadSmallMap (void) {
  //DEFINE_FUNCTION("CDfWoodsFile::ReadSmallMap()");
  bool Result;

	/* Ensure the header has been input */
  if (!m_HasHeader) {
    ErrorHandler.AddError(ERR_CUSTOM, _T("Cannot read DFWood small map data until the header has been read!"));
    return (false);
   }

	/* Attempt to allocate the data for the small map buffer */
  if (!AllocateSmallMap()) return (false);

	/* Move to the start of the small map data */
  Result = Seek(m_Header.SmallMapOffset, SEEK_SET);
  if (!Result) return (false);

	/* Input the data */
  Result = Read((char *)m_pSmallMap, m_SmallMapSize);
  return (Result);
 }
/*===========================================================================
 *		End of Class Method CDfWoodsFile::ReadSmallMap()
 *=========================================================================*/
