/*===========================================================================
 *
 * File:	Dfzcfg.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	January 18, 2003
 *
 * Description
 *
 *=========================================================================*/
#ifndef __DFZCFG_H
#define __DFZCFG_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "genfile.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Maximum number of lines parsed */
  #define DFZCFG_MAX_LINES 32

	/* Allowed line length in the Z.CFG file */
  #define DFZCFG_LINE_LENGTH (_MAX_PATH + 32)

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/

	/* Z.CFG record definition */
  typedef struct {
	char* pVariable;
	char* pValue;
   } dfzcfgrecord_t;
    
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CDFZConfig Definition
 *
 * Handles input/output of the Z.CFG file.
 *
 *=========================================================================*/
class CDFZConfig {

  /*---------- Begin Private Class Members ----------------------*/
protected:
  CGenFile	 m_File;		/* File object for I/O */
  
	/* The Z.CFG data */
  dfzcfgrecord_t m_Records[DFZCFG_MAX_LINES];
  int		 m_NumRecords;


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CDFZConfig();
  virtual void Destroy (void);

	/* Adds a new config to the end of the array */
  bool AddConfig (const char* pVariable, const char* pValue);

	/* Creates a standard config */
  bool CreateDefault (void);

	/* Attempt to find a specific variable */
  int FindRecord (const char* pVariable);
  
	/* Input/Output Z.CFG file */
  bool Load (const char* pFilename, const bool MakeDefault = true);
  bool Save (const char* pFilename);

	/* Set a specific record */
  bool SetValue (const char* pVariable, const char* pValue);

 };
/*===========================================================================
 *		End of Class CDFZConfig Definition
 *=========================================================================*/





#endif
/*===========================================================================
 *		End of File Dfzcfg.H
 *=========================================================================*/

