/*===========================================================================
 *
 * File:	Nifbase.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 * 26 October 2003
 *	- Allowed NifReadString() to input larger strings.
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "nifbase.h"
#include "math.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifBase.cpp");
  CLogFile CNifBaseRecord::OutputLog;
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBaseRecord Constructor
 *
 *=========================================================================*/
CNifBaseRecord::CNifBaseRecord () {
  //DEFINE_FUNCTION("CNifBaseRecord::CNifBaseRecord()");
  m_IsLoaded = false;
  m_NameLength = 0;
  m_EndOffset   = -1;
  m_StartOffset = -1;
 }
/*===========================================================================
 *		End of Class CNifBaseRecord Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBaseRecord Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifBaseRecord::Destroy (void) {
  //DEFINE_FUNCTION("CNifBaseRecord::Destroy()");
  m_IsLoaded = false;
  m_EndOffset   = -1;
  m_StartOffset = -1;
 }
/*===========================================================================
 *		End of Class Method CNifBaseRecord::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBaseRecord Method - bool Read (FileHandle);
 *
 *=========================================================================*/
bool CNifBaseRecord::Read (CGenFile& FileHandle) {
  //DEFINE_FUNCTION("CNifBaseRecord::Read()");

	/* Default implementation does nothing */
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifBaseRecord::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBaseRecord Method - bool Write (FileHandle);
 *
 * Virtual public method to output the record to the current position in
 * the file.  Outputs the record name size/string and returns true on success.
 *
 *=========================================================================*/
bool CNifBaseRecord::Write (CGenFile& FileHandle) {
  //DEFINE_FUNCTION("CNifBaseRecord::Write()");
  bool Result;

	/* Ignore if nothing has been loaded */
  if (!m_IsLoaded) return (false);

	/* Output the name string and length */
  Result = FileHandle.WriteLong(m_NameLength);

  if (m_NameLength > 0) {
    if (Result) Result = FileHandle.Write(m_Name, m_NameLength);
   }

  return (Result);
 }
/*===========================================================================
 *		End of Class Method CNifBaseRecord::Write()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - bool NifReadString (FileHandle, Buffer, pLength);
 *
 * Input helper method to input a string from the current location in the
 * given file.  Returns false on any error.
 *
 *=========================================================================*/
bool NifReadString (CGenFile& FileHandle, CString& Buffer, long* pLength) {
  //DEFINE_FUNCTION("NifReadString()");
  long  StringSize;
  char* pString;
  bool  Result;

	/* Empty the given string */
  Buffer.Empty();

	/* Input the string length in bytes, NUL character not included */
  Result = FileHandle.ReadLong(StringSize);
  if (!Result) return (false);
  if (pLength != NULL) *pLength = StringSize;

	/* Check for special case of empty string */
  if (StringSize == 0) return (true);

  if (StringSize > 10024 || StringSize < 0) {
    ErrorHandler.AddError(ERR_BADINPUT, "Invalid NIF string size (%ld, offset 0x%08lX)", StringSize, FileHandle.Tell());
    return (false);
   }
  
	/* Input string bytes */
  pString = Buffer.GetBufferSetLength(StringSize+1);
  ASSERT(pString != NULL);

  Result = FileHandle.Read(pString, StringSize);
  Buffer.ReleaseBuffer(StringSize);

  ASSERT(StringSize == Buffer.GetLength());
  
  return (Result);
 }
/*===========================================================================
 *		End of Function NifReadString()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - bool NifWriteString (FileHandle, Buffer);
 *
 * Input helper method to output a string to the current location in the
 * given file.  Returns false on any error.
 *
 *=========================================================================*/
bool NifWriteString (CGenFile& FileHandle, CString& Buffer) {
  //DEFINE_FUNCTION("NifWriteString()");
  bool  Result;

	/* Output the string length in bytes, NUL character not included */
  Result = FileHandle.WriteLong(Buffer.GetLength());
  if (!Result) return (false);

	/* Check for special case of empty string */
  if (Buffer.GetLength() == 0) return (true);

	/* Output string bytes */
  Result = FileHandle.Write(Buffer, Buffer.GetLength());
  return (Result);
 }
/*===========================================================================
 *		End of Function NifWriteString()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - nifpos_t NifMatrixGetRotation (const float* pMatrix);
 *
 * Computes the angle of rotation in degrees from the given 4x4 matrix as
 * in the NifNode and NifTriShape records.
 *
 *=========================================================================*/
nifpos_t NifMatrixGetRotation (const float* pMatrix) {
  DEFINE_FUNCTION("NifMatrixGetRotation()");
  nifpos_t Results;
 
	/* Ensure valid input */
  ASSERT(pMatrix != NULL);

	/* Compute the Y-Angle */
  Results.Y = (float) (asin(-pMatrix[9])*180.0/M_PI);

	/* Compute the X/Z components depending on the Y-angle */
  if (fabs(pMatrix[10]) <= 0.01 && fabs(pMatrix[11]) <= 0.01) { 
    Results.Z = (float) ((asin(pMatrix[4]) + acos(pMatrix[5]))/2.0*180.0/M_PI);
    Results.X = (float) (asin(pMatrix[4])*180.0/M_PI + Results.Z);
   }
  else {
    //Results.X = (float) (asin(pMatrix[10] / cos(Results.Y*M_PI/180.0))*180.0/M_PI);
    //Results.Z = (float) (acos(pMatrix[3]  / cos(Results.Y*M_PI/180.0))*180.0/M_PI);
    Results.Z = (float) ((asin(pMatrix[4]) + acos(pMatrix[5]))/2.0*180.0/M_PI);
    Results.X = (float) (asin(pMatrix[4])*180.0/M_PI + Results.Z);
   }

  Results.X = (float) (ceil(Results.X * 100.0)/100.0);
  Results.Y = (float) (ceil(Results.Y * 100.0)/100.0);
  Results.Z = (float) (ceil(Results.Z * 100.0)/100.0);

  return (Results);
 }
/*===========================================================================
 *		End of Function NifMatrixGetRotation()
 *=========================================================================*/
