/*===========================================================================
 *
 * File:	Nifbase.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/
#ifndef __NIFBASE_H
#define __NIFBASE_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "GenFile.h"
  #include "NifTypes.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CNifBaseRecord Definition
 *
 * Base class used to derive other NIF records from.
 *
 *=========================================================================*/
class CNifBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
protected:
  CString	m_Name;		/* Record name string */
  long		m_NameLength;
  bool		m_IsLoaded;	/* Has the record data been loaded? */

  long		m_StartOffset;
  long		m_EndOffset;


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:
  static CLogFile OutputLog;

	/* Class Constructors/Destructors */
  CNifBaseRecord();
  virtual ~CNifBaseRecord() { Destroy(); }
  virtual void Destroy (void);

	/* Get class members */
  bool		 IsLoaded       (void) const { return (m_IsLoaded); }
  bool		 IsName         (const char* pString) const { return (m_Name.CompareNoCase(pString) == 0); }
  const char*    GetName	(void) const { return (m_Name); }
  const CString& GetNameString	(void) const { return (m_Name); }
  long		 GetStartOffset (void) const { return (m_StartOffset); }
  long		 GetEndOffset   (void) const { return (m_EndOffset); }
  long		 GetSize        (void) const { return (m_EndOffset - m_StartOffset + 1); }
 
	/* Used to identify record to user */
  virtual const char* GetDisplayName (void) const { return (m_Name); }
  virtual const char* GetRecordName  (void) const { return (""); }

	/* Set class members */
  void SetIsLoaded    (const bool Flag)     { m_IsLoaded = Flag; }
  void SetName        (const char* pString) { m_Name = pString; }
  void SetNameLength  (const int Length)    { m_NameLength = Length; }
  void SetStartOffset (const long Offset)   { m_StartOffset = Offset; }
  void SetEndOffset   (const long Offset)   { m_EndOffset   = Offset; }

	/* Input output methods */
  virtual bool Read  (CGenFile& FileHandle);
  virtual bool Write (CGenFile& FileHandle);

 };
/*===========================================================================
 *		End of Class CNifBaseRecord Definition
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Function Prototypes
 *
 *=========================================================================*/

	/* Output/Input string helper method */
  bool NifReadString  (CGenFile& FileHandle, CString& Buffer, long* pLength = NULL);
  bool NifWriteString (CGenFile& FileHandle, CString& Buffer);

	/* Matrix functions */
  nifpos_t NifMatrixGetRotation (const float* pMatrix);
  
/*===========================================================================
 *		End of Function Prototypes
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Nifbase.H
 *=========================================================================*/
