/*===========================================================================
 *
 * File:	NifBillboardNode.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "NifBillboardNode.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifBillboardNode.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBillboardNode Constructor
 *
 *=========================================================================*/
CNifBillboardNode::CNifBillboardNode () {
  //DEFINE_FUNCTION("CNifBillboardNode::CNifBillboardNode()");

 }
/*===========================================================================
 *		End of Class CNifBillboardNode Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBillboardNode Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifBillboardNode::Destroy (void) {
  DEFINE_FUNCTION("CNifBillboardNode::Destroy()");

	/* Call the base class method */
  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifBillboardNode::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifBillboardNode Method - bool Read (FileHandle);
 *
 * Input the node data from the current position in the given file.  Assumes
 * that the current position in the file is the byte after the last character
 * in the "NifNode" name record header.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifBillboardNode::Read (CGenFile& FileHandle) {
  bool Result;

	/* Delete the current record contents */
  Destroy();

	/* Input the particle string */
  Result = NifReadString(FileHandle, m_NodeName);
  if (!Result) return (false);

  	/* Input first three numbers */
  Result = FileHandle.ReadLong(m_Unknown1);
  if (Result) Result = FileHandle.ReadLong(m_Unknown2);
  if (Result) Result = FileHandle.ReadShort(m_Unknown3);
  if (!Result) return (false);

	/* Input values array */
  Result = FileHandle.Read((char *)&m_Values[0], sizeof(float)*NIF_BILLBOARDNODE_NUMVALUES);
  if (Result) Result = m_Array1.Read(FileHandle);
  if (Result) Result = m_Array2.Read(FileHandle);
  if (Result) Result = m_Array3.Read(FileHandle);
  if (Result) Result = m_Array4.Read(FileHandle);
  if (!Result) return (false);

	/* Return success */
  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifBillboardNode::Read()
 *=========================================================================*/
  

/*===========================================================================
 *
 * Class CNifBillboardNode Method - bool Write (FileHandle);
 *
 * Input the node data to the current position in the given file.  Outputs the
 * record header and data.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifBillboardNode::Write (CGenFile& FileHandle) {
  bool Result;

	/* Call the base class method first */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

	/* Output the particle name string */
  Result = NifWriteString(FileHandle, m_NodeName);
  if (!Result) return (false);

  	/* Output first three numbers */
  Result = FileHandle.WriteLong(m_Unknown1);
  if (Result) Result = FileHandle.WriteLong(m_Unknown2);
  if (Result) Result = FileHandle.WriteShort(m_Unknown3);
  if (!Result) return (false);

	/* Output values array */
  Result = FileHandle.Write((char *)&m_Values[0], sizeof(float)*NIF_BILLBOARDNODE_NUMVALUES);
  if (Result) Result = m_Array1.Write(FileHandle);
  if (Result) Result = m_Array2.Write(FileHandle);
  if (Result) Result = m_Array3.Write(FileHandle);
  if (Result) Result = m_Array4.Write(FileHandle);
  if (!Result) return (false);

	/* Return success */
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifBillboardNode::Write()
 *=========================================================================*/

