/*===========================================================================
 *
 * File:	NifDirectionalLight.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "NifDirectionalLight.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifDirectionalLight.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifDirectionalLight Constructor
 *
 *=========================================================================*/
CNifDirectionalLight::CNifDirectionalLight () {
  //DEFINE_FUNCTION("CNifDirectionalLight::CNifDirectionalLight()");
  m_LightName = _T(""); 
 }
/*===========================================================================
 *		End of Class CNifDirectionalLight Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifDirectionalLight Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifDirectionalLight::Destroy (void) {
  //DEFINE_FUNCTION("CNifDirectionalLight::Destroy()");
  m_LightName = _T("");

	/* Call the base class method */
  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifDirectionalLight::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifDirectionalLight Method - bool Read (FileHandle);
 *
 * Input the node data from the current position in the given file.  Assumes
 * that the current position in the file is the byte after the last character
 * in the "NifNode" name record header.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifDirectionalLight::Read (CGenFile& FileHandle) {
  bool Result;
  
	/* Delete the current record contents */ 
  Destroy();

	/* Input the optional texture string name */
  Result = NifReadString(FileHandle, m_LightName);
  if (!Result) return (false);
  
	/* Input first three numbers */
  Result = FileHandle.ReadLong(m_Unknown1);
  if (Result) Result = FileHandle.ReadLong(m_Unknown2);
  if (Result) Result = FileHandle.ReadShort(m_Unknown3);
  if (Result) Result = FileHandle.Read((char *)&m_Floats[0], sizeof(float)*NIF_DIRLIGHT_NUMFLOATS);
  if (!Result) return (false);

	/* Return success */
  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifDirectionalLight::Read()
 *=========================================================================*/
  

/*===========================================================================
 *
 * Class CNifDirectionalLight Method - bool Write (FileHandle);
 *
 * Input the node data to the current position in the given file.  Outputs the
 * record header and data.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifDirectionalLight::Write (CGenFile& FileHandle) {
  bool Result;

	/* Call the base class method first */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

	/* Output the optional texture string name */
  Result = NifWriteString(FileHandle, m_LightName);
  if (!Result) return (false);
  
	/* Output first three numbers */
  Result = FileHandle.WriteLong(m_Unknown1);
  if (Result) Result = FileHandle.WriteLong(m_Unknown2);
  if (Result) Result = FileHandle.WriteShort(m_Unknown3);
  if (Result) Result = FileHandle.Write((char *)&m_Floats[0], sizeof(float)*NIF_DIRLIGHT_NUMFLOATS);
  if (!Result) return (false);

	/* Return success */
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifDirectionalLight::Write()
 *=========================================================================*/
