/*===========================================================================
 *
 * File:	NifFloatArray.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Monday, December 23, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "nifFloatArray.h"


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  DEFINE_FILE("NifFloatArray.cpp");
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFloatArray Constructor
 *
 *=========================================================================*/
CNifFloatArray::CNifFloatArray () {
  //DEFINE_FUNCTION("CNifFloatArray::CNifFloatArray()");
  m_Size = 0;
  m_AllocatedSize = 0;
  m_pData = NULL;
  m_Options = 0;
  m_FixedSize = 0;
 }
/*===========================================================================
 *		End of Class CNifFloatArray Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFloatArray Method - void Destroy (void);
 *
 * Description
 *
 *=========================================================================*/
void CNifFloatArray::Destroy (void) {
  DEFINE_FUNCTION("CNifFloatArray::Destroy()");

  DestroyArrayPointer(m_pData);
  m_Size = 0;
  m_AllocatedSize = 0;
 }
/*===========================================================================
 *		End of Class Method CNifFloatArray::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFloatArray Method - bool Read (CGenFile& FileHandle);
 *
 * Description
 *
 *=========================================================================*/
bool CNifFloatArray::Read (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifFloatArray::Read()");
  bool Result;

  Destroy();

	/* Input the long array size value if required */
  if (m_Options & NIFFLOAT_OPTION_SHORTSIZE) {
    short sTemp;
    Result = FileHandle.ReadShort(sTemp);
    m_Size = sTemp;
   }
  else
    Result = FileHandle.ReadLong(m_Size);

  if (!Result) return (false);

	/* Ensure a valid input size */
  if ((m_Options & NIFFLOAT_OPTION_FIXEDSIZE) == 0 && m_Size < 0) {
    ErrorHandler.AddError(ERR_CUSTOM, "Invalid array size input (%ld)!", m_Size);
    return (false);
   }

  if (m_Size == 0) {
    m_AllocatedSize = 0;
    return (true);
   }

	/* Allocate the array */
  if (m_Options & NIFFLOAT_OPTION_FIXEDSIZE) {
    CreateArrayPointer(m_pData, float, m_FixedSize);
    m_AllocatedSize = m_FixedSize;
    Result = FileHandle.Read((char *)m_pData, sizeof(float)*m_FixedSize);
   }
  else {
    CreateArrayPointer(m_pData, float, m_Size);
    m_AllocatedSize = m_Size;
    Result = FileHandle.Read((char *)m_pData, sizeof(float)*m_Size);
   }
  
  return (Result);
 }
/*===========================================================================
 *		End of Class Method CNifFloatArray::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifFloatArray Method - bool Write (CGenFile& FileHandle);
 *
 * Description
 *
 *=========================================================================*/
bool CNifFloatArray::Write (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifFloatArray::Write()");
  bool Result;

	/* Output the long array size value */
  if (m_Options & NIFFLOAT_OPTION_SHORTSIZE)
    Result = FileHandle.WriteShort((short)m_Size); 
  else
    Result = FileHandle.WriteLong(m_Size); 

  if (!Result) return (false);

	/* Ensure a valid input size */
  if (m_Size == 0 || m_pData == NULL) return (true);

  if (m_Options & NIFFLOAT_OPTION_FIXEDSIZE)
    Result = FileHandle.Write((char *)m_pData, sizeof(long)*m_FixedSize);
  else
    Result = FileHandle.Write((char *)m_pData, sizeof(long)*m_Size);

  return (Result);
 }
/*===========================================================================
 *		End of Class Method CNifFloatArray::Write()
 *=========================================================================*/
