/*===========================================================================
 *
 * File:	NifKeyFrameData.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Monday, December 23, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "NifKeyFrameData.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifKeyFrameData.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifKeyFrameDataArray Constructor
 *
 *=========================================================================*/
CNifKeyFrameDataArray::CNifKeyFrameDataArray () {
  m_pData       = NULL;
  m_Size        = 0;
  m_RecordSize  = 1;
  m_ControlType = 0;
  m_Type        = 0;
 }
/*===========================================================================
 *		End of Class CNifKeyFrameDataArray Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifKeyFrameDataArray Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifKeyFrameDataArray::Destroy (void) {
  DEFINE_FUNCTION("CNifKeyFrameDataArray::Destroy()");
  DestroyArrayPointer(m_pData);
  m_Size = 0;
 }
/*===========================================================================
 *		End of Class Method CNifKeyFrameDataArray::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifKeyFrameDataArray Method - bool CheckKey (Index);
 *
 * Checks the given key index to ensure it is valid. Returns false and
 * logs an error message if it is invalid.
 *
 *=========================================================================*/
bool CNifKeyFrameDataArray::CheckKey (const int Index) {
 
  if (IsValidKey(Index)) return (true);

  ErrorHandler.AddError(ERR_BADINPUT, _T("Invalid key index %d!"), Index);
  return (false);
 }
/*===========================================================================
 *		End of Class Method CNifKeyFrameDataArray::CheckKey()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifKeyFrameDataArray Method - long FindRecordSize (ControlType);
 *
 * Returns the estimated record size of the keys for the current array 
 * type with the given controller type. Record size is given in 4 byte
 * sizes (float/long). Returns a 0 result on any error.
 *
 *=========================================================================*/
long CNifKeyFrameDataArray::FindRecordSize (const int ControlType) {

  switch (m_Type) {
    case NIF_KF_ROTATION: {
	switch (ControlType) {
	  case 1: return (5);
	  case 3: return (8);
	  case 5: return (8);
	  default:
		ErrorHandler.AddError(ERR_BADINPUT, _T("Unknown rotation controller type %d!"), ControlType);
		return (0);
	 }
     }
    case NIF_KF_POSITION: {
	switch (m_ControlType) {
	  case 1: return (4);
	  case 3: return (7);
	  case 2: return (10);
	  default: 
		ErrorHandler.AddError(ERR_BADINPUT, _T("Unknown position controller type %d!"), ControlType);
		return (0);
	}
     }
    case NIF_KF_SCALE: {
	switch (m_ControlType) {
	  case 1: return (2);
	  case 3: return (5);
	  case 2: return (4);
	  default: 
		ErrorHandler.AddError(ERR_BADINPUT, _T("Unknown scale controller type %d!"), ControlType);
		return (0);
	}
      }
   }
  
  ErrorHandler.AddError(ERR_BADINPUT, _T("Unknown keyframe type %d!"), m_Type);
  return (0);
 }
/*===========================================================================
 *		End of Class Method CNifKeyFrameDataArray::FindRecordSize()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifKeyFrameDataArray Method - nifkfroteul_t* GetKeyRotEul (Index);
 *
 * Attempts to return the given euler rotation key data. Returns NULL
 * in the key index is invalid or the key type does not match.
 *
 *=========================================================================*/
nifkfroteul_t* CNifKeyFrameDataArray::GetKeyRotEul (const int Index) {

	/* Ensure the correct type of data */
  if (!IsRotArray() || !IsEulData()) {
    ErrorHandler.AddError(ERR_BADINPUT, _T("Keyframe data does not contain rotation euler keys!"));
    return (NULL);
   }

	/* Ensure a valid key index */
  if (!CheckKey(Index)) return (NULL);
 
	/* Return the requested key pointer */
  return &((nifkfroteul_t *) m_pData)[Index];  
 }
/*===========================================================================
 *		End of Class Method CNifKeyFrameDataArray::GetKeyRotEul()
 *=========================================================================*/


nifkfrotlin_t* CNifKeyFrameDataArray::GetKeyRotLin (const int Index) {

	/* Ensure the correct type of data */
  if (!IsRotArray() || !IsLinData()) {
    ErrorHandler.AddError(ERR_BADINPUT, _T("Keyframe data does not contain rotation linear keys!"));
    return (NULL);
   }

	/* Ensure a valid key index */
  if (!CheckKey(Index)) return (NULL);
 
	/* Return the requested key pointer */
  return &((nifkfrotlin_t *) m_pData)[Index];  
 }


nifkfrottcb_t* CNifKeyFrameDataArray::GetKeyRotTcb (const int Index) {

	/* Ensure the correct type of data */
  if (!IsRotArray() || !IsTcbData()) {
    ErrorHandler.AddError(ERR_BADINPUT, _T("Keyframe data does not contain rotation TCB keys!"));
    return (NULL);
   }

	/* Ensure a valid key index */
  if (!CheckKey(Index)) return (NULL);
 
	/* Return the requested key pointer */
  return &((nifkfrottcb_t *) m_pData)[Index];  
 }


nifkfrotunk_t* CNifKeyFrameDataArray::GetKeyRotUnk (const int Index) {

	/* Ensure the correct type of data */
  if (!IsRotArray() || !IsUnkData()) {
    ErrorHandler.AddError(ERR_BADINPUT, _T("Keyframe data does not contain rotation unknown keys!"));
    return (NULL);
   }

	/* Ensure a valid key index */
  if (!CheckKey(Index)) return (NULL);
 
	/* Return the requested key pointer */
  return &((nifkfrotunk_t *) m_pData)[Index];  
 }


nifkfposlin_t* CNifKeyFrameDataArray::GetKeyPosLin (const int Index) {

	/* Ensure the correct type of data */
  if (!IsPosArray() || !IsLinData()) {
    ErrorHandler.AddError(ERR_BADINPUT, _T("Keyframe data does not contain position linear keys!"));
    return (NULL);
   }

	/* Ensure a valid key index */
  if (!CheckKey(Index)) return (NULL);
 
	/* Return the requested key pointer */
  return &((nifkfposlin_t *) m_pData)[Index];  
 }


nifkfpostcb_t* CNifKeyFrameDataArray::GetKeyPosTcb (const int Index) {

	/* Ensure the correct type of data */
  if (!IsPosArray() || !IsTcbData()) {
    ErrorHandler.AddError(ERR_BADINPUT, _T("Keyframe data does not contain position TCB keys!"));
    return (NULL);
   }

	/* Ensure a valid key index */
  if (!CheckKey(Index)) return (NULL);
 
	/* Return the requested key pointer */
  return &((nifkfpostcb_t *) m_pData)[Index];  
 }


nifkfposbez_t* CNifKeyFrameDataArray::GetKeyPosBez (const int Index) {

	/* Ensure the correct type of data */
  if (!IsPosArray() || !IsBezData()) {
    ErrorHandler.AddError(ERR_BADINPUT, _T("Keyframe data does not contain position bezier keys!"));
    return (NULL);
   }

	/* Ensure a valid key index */
  if (!CheckKey(Index)) return (NULL);
 
	/* Return the requested key pointer */
  return &((nifkfposbez_t *) m_pData)[Index];  
 }


nifkfscalin_t* CNifKeyFrameDataArray::GetKeyScaLin (const int Index) {

	/* Ensure the correct type of data */
  if (!IsScaArray() || !IsLinData()) {
    ErrorHandler.AddError(ERR_BADINPUT, _T("Keyframe data does not contain scale linear keys!"));
    return (NULL);
   }

	/* Ensure a valid key index */
  if (!CheckKey(Index)) return (NULL);
 
	/* Return the requested key pointer */
  return &((nifkfscalin_t *) m_pData)[Index];  
 }


nifkfscatcb_t* CNifKeyFrameDataArray::GetKeyScaTcb (const int Index) {

	/* Ensure the correct type of data */
  if (!IsScaArray() || !IsTcbData()) {
    ErrorHandler.AddError(ERR_BADINPUT, _T("Keyframe data does not contain scale TCB keys!"));
    return (NULL);
   }

	/* Ensure a valid key index */
  if (!CheckKey(Index)) return (NULL);
 
	/* Return the requested key pointer */
  return &((nifkfscatcb_t *) m_pData)[Index];  
 }


nifkfscabez_t* CNifKeyFrameDataArray::GetKeyScaBez (const int Index) {

	/* Ensure the correct type of data */
  if (!IsScaArray() || !IsBezData()) {
    ErrorHandler.AddError(ERR_BADINPUT, _T("Keyframe data does not contain scale bezier keys!"));
    return (NULL);
   }

	/* Ensure a valid key index */
  if (!CheckKey(Index)) return (NULL);
 
	/* Return the requested key pointer */
  return &((nifkfscabez_t *) m_pData)[Index];  
 }


/*===========================================================================
 *
 * Class CNifKeyFrameDataArray Method - bool Read (FileHandle);
 *
 *=========================================================================*/
bool CNifKeyFrameDataArray::Read (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifKeyFrameDataArray::Read()");
  bool Result;

  Destroy();

	/* Input size */
  //SystemLog.Printf ("\t\tKeyFrameDataArrayOffset = 0x%08lX", FileHandle.Tell());
  Result = FileHandle.ReadLong(m_Size);
  if (!Result) return (false);

	/* Check for 0 sized array */
  if (m_Size == 0) return (true);

	/* Input controller type and compute the record size */
  Result = FileHandle.ReadLong(m_ControlType);
  if (!Result) return (false);

  m_RecordSize = FindRecordSize(m_ControlType);
  if (m_RecordSize < 0) return (false);
  
	/* Allocate and input the array */
  CreateArrayPointer(m_pData, float, sizeof(float)*m_Size*m_RecordSize);
  Result = FileHandle.Read((char *)m_pData, sizeof(float)*m_Size*m_RecordSize);

  return (Result);
 }
/*===========================================================================
 *		End of Class Method CNifKeyFrameDataArray::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifKeyFrameDataArray Method - bool Write (FileHandle);
 *
 *=========================================================================*/
bool CNifKeyFrameDataArray::Write (CGenFile& FileHandle) {
  DEFINE_FUNCTION("CNifKeyFrameDataArray::Write()");
  bool Result;

  	/* Output size */
  Result = FileHandle.WriteLong(m_Size);
  if (!Result) return (false);

	/* Check for 0 sized array */
  if (m_Size == 0) return (true);

	/* Output unknown value */
  Result = FileHandle.WriteLong(m_ControlType);
  if (!Result) return (false);

	/* Allocate and input the array */
  if (m_Size > 0 && m_pData != NULL) {
    Result = FileHandle.Write((char *)m_pData, sizeof(float)*m_Size*m_RecordSize);
   }

  return (Result);
 }
/*===========================================================================
 *		End of Class Method CNifKeyFrameDataArray::Write()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifKeyFrameData Constructor
 *
 *=========================================================================*/
CNifKeyFrameData::CNifKeyFrameData () {
  //DEFINE_FUNCTION("CNifKeyFrameData::CNifKeyFrameData()");
  m_RotArray.SetType(NIF_KF_ROTATION);
  m_ScaArray.SetType(NIF_KF_SCALE);
  m_PosArray.SetType(NIF_KF_POSITION);
 }
/*===========================================================================
 *		End of Class CNifKeyFrameData Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifKeyFrameData Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifKeyFrameData::Destroy (void) {
  //DEFINE_FUNCTION("CNifKeyFrameData::Destroy()");
  m_RotArray.Destroy();
  m_PosArray.Destroy();
  m_ScaArray.Destroy();
  
  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifKeyFrameData::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifKeyFrameData Method - bool Read (FileHandle);
 *
 * Input the node data from the current position in the given file.  Assumes
 * that the current position in the file is the byte after the last character
 * in the "NifNode" name record header.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifKeyFrameData::Read (CGenFile& FileHandle) {
  bool Result;
  
	/* Delete the current record contents */ 
  Destroy();
  
  Result = m_RotArray.Read(FileHandle);
  if (Result) Result = m_PosArray.Read(FileHandle);
  if (Result) Result = m_ScaArray.Read(FileHandle);
  if (!Result) return (false);

	/* Return success */
  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifKeyFrameData::Read()
 *=========================================================================*/
  

/*===========================================================================
 *
 * Class CNifKeyFrameData Method - bool Write (FileHandle);
 *
 * Input the node data to the current position in the given file.  Outputs the
 * record header and data.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifKeyFrameData::Write (CGenFile& FileHandle) {
  bool Result;

	/* Call the base class method first */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

  Result = m_RotArray.Write(FileHandle);
  if (Result) Result = m_PosArray.Write(FileHandle);
  if (Result) Result = m_ScaArray.Write(FileHandle);
  if (!Result) return (false);

	/* Return success */
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifKeyFrameData::Write()
 *=========================================================================*/



