/*===========================================================================
 *
 * File:	Nifnode.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "nifnode.h"



/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("NifNode.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifNode Constructor
 *
 *=========================================================================*/
CNifNode::CNifNode () {
  //DEFINE_FUNCTION("CNifNode::CNifNode()");

  m_Unknown1 = 0xFFFFFFFF;
  m_Unknown2 = 0xFFFFFFFF;
  m_Unknown3 = 0;

  m_Array2.SetOptions(NIFFLOAT_OPTION_FIXEDSIZE | NIFFLOAT_OPTION_SIZEFLAGGED);
  m_Array2.SetSize(NIF_NODE_NUMPOINTS);
 }
/*===========================================================================
 *		End of Class CNifNode Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifNode Method - void Destroy (void);
 *
 *=========================================================================*/
void CNifNode::Destroy (void) {
  DEFINE_FUNCTION("CNifNode::Destroy()");

  m_Array1.Destroy();
  m_Array2.Destroy();
  m_ChildArray.Destroy();
  m_Array4.Destroy();

	/* Call the base class method */
  CNifBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CNifNode::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifNode Method - void ModPos (Flags, Delta);
 *
 * Description
 *
 *=========================================================================*/
void CNifNode::ModPos (const int Flags, const float Delta) {
  if (Flags & NIF_MODX) m_Points[0] += Delta;
  if (Flags & NIF_MODY) m_Points[1] += Delta;
  if (Flags & NIF_MODZ) m_Points[2] += Delta;
 }
/*===========================================================================
 *		End of Class Method CNifNode::ModPos()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifNode Method - void ModPosWorld (Flags, Delta);
 *
 * Mods the node's position by the given amount along with XYZ world axes
 * (rotated according to the node parameters).
 *
 *=========================================================================*/
void CNifNode::ModPosWorld (const int Flags, const float Delta) {
  nifpos_t XDelta;
  nifpos_t YDelta;
  nifpos_t ZDelta;

	/* Mod the position */
  if (Flags & NIF_MODX) {
    XDelta.X = m_Points[3]  * Delta;
    XDelta.Y = m_Points[4]  * Delta;
    XDelta.Z = m_Points[5]  * Delta;
  
    m_Points[0] += XDelta.X;
    m_Points[1] += XDelta.Y;
    m_Points[2] += XDelta.Z;
   }

  if (Flags & NIF_MODY) {
    YDelta.X = m_Points[6]  * Delta;
    YDelta.Y = m_Points[7]  * Delta;
    YDelta.Z = m_Points[8]  * Delta;

    m_Points[0] += YDelta.X;
    m_Points[1] += YDelta.Y;
    m_Points[2] += YDelta.Z;
   }

  if (Flags & NIF_MODZ) {
    ZDelta.X = m_Points[9]  * Delta;
    ZDelta.Y = m_Points[10] * Delta;
    ZDelta.Z = m_Points[11] * Delta;
    
    m_Points[0] += ZDelta.X;
    m_Points[1] += ZDelta.Y;
    m_Points[2] += ZDelta.Z;
   }
 }
/*===========================================================================
 *		End of Class Method CNifNode::ModPosWorld()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CNifNode Method - bool Read (FileHandle);
 *
 * Input the node data from the current position in the given file.  Assumes
 * that the current position in the file is the byte after the last character
 * in the "NifNode" name record header.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifNode::Read (CGenFile& FileHandle) {
  bool Result;

	/* Delete the current record contents */
  Destroy();

	/* Input the node name */
  Result = NifReadString(FileHandle, m_Name);
  if (!Result) return (false);

	/* Input first three numbers */
  Result = FileHandle.ReadLong(m_Unknown1);
  if (Result) Result = FileHandle.ReadLong(m_Unknown2);
  if (Result) Result = FileHandle.ReadShort(m_Unknown3);
  if (!Result) return (false);

	/* Input axis coordinates */
  Result = FileHandle.Read((char *)&m_Points[0], sizeof(float)*NIF_NODE_NUMPOINTS);
  if (!Result) return (false);

	/* Input index count */
  Result = m_Array1.Read(FileHandle);
  if (Result) Result = m_Array2.Read(FileHandle);
  if (Result) Result = m_ChildArray.Read(FileHandle);
  if (Result) Result = m_Array4.Read(FileHandle);
  if (!Result) return (false);

	/* Return success */
  SetIsLoaded(true);
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifNode::Read()
 *=========================================================================*/
  

/*===========================================================================
 *
 * Class CNifNode Method - bool Write (FileHandle);
 *
 * Input the node data to the current position in the given file.  Outputs the
 * record header and data.  Returns false on any error.
 *
 *=========================================================================*/
bool CNifNode::Write (CGenFile& FileHandle) {
  bool Result;

	/* Call the base class method first */
  Result = CNifBaseRecord::Write(FileHandle);
  if (!Result) return (false);

	/* Output the node name */
  Result = NifWriteString(FileHandle, m_Name);
  if (!Result) return (false);

	/* Output first three numbers */
  Result = FileHandle.WriteLong(m_Unknown1);
  if (Result) Result = FileHandle.WriteLong(m_Unknown2);
  if (Result) Result = FileHandle.WriteShort(m_Unknown3);
  if (!Result) return (false);

	/* Output axis coordinates */
  Result = FileHandle.Write((char *)&m_Points[0], sizeof(float)*NIF_NODE_NUMPOINTS);
  if (!Result) return (false);

  Result = m_Array1.Write(FileHandle);
  if (Result) Result = m_Array2.Write(FileHandle);
  if (Result) Result = m_ChildArray.Write(FileHandle);
  if (Result) Result = m_Array4.Write(FileHandle);
  if (!Result) return (false);
	/* Return success */
  return (true);
 }
/*===========================================================================
 *		End of Class Method CNifNode::Write()
 *=========================================================================*/

