/*===========================================================================
 *
 * File:	Niftrishape.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/
#ifndef __NIFTRISHAPE_H
#define __NIFTRISHAPE_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "NifBase.h"
  #include "NifLongArray.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Number of coordinates in the record data */
  #define NIF_TRISHAPE_NUMCOORS   16
  #define NIF_TRISHAPE_MAXINDICES 64

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CNifTriShape Definition
 *
 * Description
 *
 *=========================================================================*/
class CNifTriShape : public CNifBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CString	m_ShapeName;		/* Name string */

  long		m_Unknown1;		/* Unknown values */
  long		m_Unknown2;
  short		m_Unknown3;

  float		m_Coordinates[NIF_TRISHAPE_NUMCOORS];	/* Array of coordinates */

  CNifLongArray	m_Array;		/* List of indices */
  long		m_Unknown4;
  long		m_TriShapeDataIndex;
  long		m_Unknown6;


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CNifTriShape();
  virtual ~CNifTriShape() { Destroy(); }
  virtual void Destroy (void);

  	/* Return the optional record name */
  virtual const char*  GetRecordName  (void) const { return (m_ShapeName); }

	/* Get class members */
  float		 GetFloat	      (const int Index) const	{ return ((Index >= 0 && Index < NIF_TRISHAPE_NUMCOORS) ? m_Coordinates[Index] : 0); }
  float*	 GetFloatData	      (void)			{ return &m_Coordinates[0]; }
  long		 GetTriShapeDataIndex (void) const		{ return (m_TriShapeDataIndex); }
  CNifLongArray* GetMaterialArray     (void)			{ return (&m_Array); }

	/* Modify the trishape position */
  void ModPos   (const int Flags, const float Delta);
  void ModScale (const float Delta);

	/* Input/Output the section from the current position in the file */
  virtual bool Read  (CGenFile& FileHandle);
  virtual bool Write (CGenFile& FileHandle);

	/* Set class members */
  void SetFloat (const int Index, const float Value) { if (Index >= 0 && Index < NIF_TRISHAPE_NUMCOORS) m_Coordinates[Index] = Value; }


 };
/*===========================================================================
 *		End of Class CNifTriShape Definition
 *=========================================================================*/



#endif
/*===========================================================================
 *		End of File Niftrishape.H
 *=========================================================================*/
