/*===========================================================================
 *
 * File:	Niftrishapedata.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	December 11, 2002
 *
 * Description
 *
 *=========================================================================*/
#ifndef __NIFTRISHAPEDATA_H
#define __NIFTRISHAPEDATA_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "NifBase.h"
  #include "NifRootCollisonNode.h"
  #include "NifShortArray.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

  #define NIF_TRISHAPE_NUMUNKNOWN3 4	/* Float */
  #define NIF_TRISHAPE_NUMUNKNOWN4 5	/* Short */

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)

#pragma pack(pop)
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CNifTriShapeData Definition
 *
 * Description
 *
 *=========================================================================*/
class CNifTriShapeData : public CNifBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  short			m_NumPoints;		/* Number of points in tri-shape */
  float			m_Unknown1;

  nifpos_t*		m_pPoints;		/* Array of points */

  float			m_Unknown2;

  nifpos_t*		m_pNormals;		/* Point normals */

  float			m_Unknown3[NIF_TRISHAPE_NUMUNKNOWN3];
  short			m_Unknown4[NIF_TRISHAPE_NUMUNKNOWN4];

  nifcolor4_t*		m_pVertexColors;
  float*		m_pUnknownPoints2;

  short			m_NumFaces;		/* Number of faces/point indices */
  long			m_NumFaceIndices;
  short*		m_pFaceIndices;

  short			m_NumRecords;
  CNifShortArray*	m_pRecords;

  nifpos_t		m_MinPosition;		/* Computed statistics */
  nifpos_t		m_MaxPosition;
  bool			m_CalcMinPosition;
  bool			m_CalcMaxPosition;


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Allocation helper methods */
  void AllocatePoints	      (void);
  void AllocateFaceIndices    (void);
  void AllocateVertexColors   (void);
  void AllocateUnknownPoints2 (void);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CNifTriShapeData();
  virtual ~CNifTriShapeData() { Destroy(); }
  virtual void Destroy (void);

	/* Get min/max position information */
  void		GetMinPos	  (nifpos_t& Position);
  void		GetMaxPos	  (nifpos_t& Position);
  int		GetNumPoints	  (void) const { return (m_NumPoints); }
  int		GetNumFaces	  (void) const { return (m_NumFaces); } 
  bool		IsValidPoint      (const int Index) { return (Index >= 0 && Index < GetNumPoints()); }
  bool		IsValidFace       (const int Index) { return (Index >= 0 && Index < GetNumFaces()); }
  bool		IsValidFaceIndice (const int Index) { return (Index >= 0 && Index < m_NumFaceIndices); }
  nifpos_t*	GetPoint  	  (const int Index) { return (IsValidPoint(Index) ? & m_pPoints[Index]        : NULL); }
  nifpos_t*	GetNormal  	  (const int Index) { return (IsValidPoint(Index) && HasNormals() ? & m_pNormals[Index] : NULL); }
  short*	GetFace		  (const int Index) { return (IsValidFaceIndice(Index) ? & m_pFaceIndices[Index] : NULL); }
  float		GetUVPoint        (const int Index) { return ((IsValidPoint(Index/2) && m_pUnknownPoints2 != NULL) ? m_pUnknownPoints2[Index] : 0); }
  float*	GetUVPointPtr     (const int Index) { return ((IsValidPoint(Index)   && m_pUnknownPoints2 != NULL) ? &m_pUnknownPoints2[Index*2] : NULL); }
  nifcolor4_t*  GetVertexColors   (void) { return (m_pVertexColors); }
  bool		HasVertexColors   (void) { return (m_Unknown4[1] != 0); }

	/* Returns true if the shape has texture information */
  bool HasTexture (void) const { return (m_pUnknownPoints2 != NULL); }
  bool HasNormals (void) const { return (m_Unknown2 != 0); }
  
	/* Modify all coordinates in shape */
  void ModPos (const int Flags, const float Delta); 

	/* Input/Output the section from the current position in the file */
  virtual bool Read  (CGenFile& FileHandle);
  virtual bool Write (CGenFile& FileHandle);

 };
/*===========================================================================
 *		End of Class CNifTriShapeData Definition
 *=========================================================================*/



#endif
/*===========================================================================
 *		End of File Niftrishapedata.H
 *=========================================================================*/

