/*===========================================================================
 *
 * File:	Mwesmfile.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Defines the CMwEsmFile class for handling a single Morrowind ESM
 * plugin file.
 *
 *=========================================================================*/
#ifndef __MWESMFILE_H
#define __MWESMFILE_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "mwesm_baserec.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Initial size of base records array */
  #define MWESM_DEFAULT_NUMRECORDS 1024

	/* Amount to increase records array when required */
  #define MWESM_ARRAY_RESIZE_FACTOR 2

	/* Used to define the base record definition array */
  #define MWESM_RECORD_DEF(Name, Func) { Name, Func }, 

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/

	/* Associates base record types with the appropriate class */
  typedef struct {
    char		Name[MWESM_HEADERNAME_SIZE+1];
    MWESMCREATE_FUNC	pfnCreate;
   } mwesmbaserecdef_t;


	/* Callback function type definition used when loading files */
  typedef int (*MWESM_LOADCALLBACK) (const mwesmbaseheader_t& Header, const float PercentLoaded);

	/* Callback function used in the ForEachRecord() method */
  typedef int (*MWESM_FOREACHCALLBACK) (CMwEsmBaseRecord* pRecord, void* pUserData);

/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CMwEsmFile Definition
 *
 * Encapsulates a single Morrowind ESM plugin file. Derived from the 
 * generic CGenFile class for handling file I/O.
 *
 *=========================================================================*/
class CMwEsmFile : public CGenFile {

  /*---------- Begin Protected Class Members --------------------*/
protected:
  CMwEsmBaseRecord**	m_ppRecords;	/* Array of records */
  int			m_NumRecords;
  int			m_AllocatedRecords; 

  bool			m_SkipData;	/* Do not input the record data */
  bool		        m_SkipBaseData;

  MWESM_LOADCALLBACK	m_fnLoadCallback;	/* Optional load ESM callback function */
  


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Attempt to add the given record to the end of the record array */
  void AddRecord (CMwEsmBaseRecord* pNewRecord);

	/* Delete all allocated records */
  void DeleteRecords (void);

	/* Attempt to resize the array of records */
  bool ResizeRecordsArray (const int NewSize);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CMwEsmFile();
  virtual ~CMwEsmFile() { Destroy(); }
  virtual void Destroy (void);

	/* Calls the given function with each currently defined record */
  int ForEachRecord (MWESM_FOREACHCALLBACK fnCallback, void* pUserData);

	/* Get class members */
  int		    GetNumRecords (void) const            { return (m_NumRecords); }
  CMwEsmBaseRecord* GetRecord	  (const int Index)       { return (IsValidRecord(Index) ? m_ppRecords[Index] : NULL); }
  bool		    IsValidRecord (const int Index) const { return (Index >= 0 && Index < m_NumRecords); }

	/* Attempt to input an ESM file */
  virtual bool LoadESM (const char* pFilename);
  virtual bool LoadESM (const char* pFilename, MWESM_LOADCALLBACK CallbackFunc);

	/* Output item info to the given file */
  virtual bool OutputItems (CGenFile& File);

	/* Attempt to output an ESM file */
  virtual bool SaveESM (const char* pFilename);

	/* Set class members */
  void SetSkipData     (const bool Flag) { m_SkipData     = Flag; }
  void SetSkipBaseData (const bool Flag) { m_SkipBaseData = Flag; }


 };
/*===========================================================================
 *		End of Class CMwEsmFile Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Mwesmfile.H
 *=========================================================================*/
