/*===========================================================================
 *
 * File:	Mwesm_baserec.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "mwesm_baserec.h"
#include "string.h"
#include "assert.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("MwESM_BaseRec.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmBaseRecord Constructor
 *
 *=========================================================================*/
CMwEsmBaseRecord::CMwEsmBaseRecord () {
  m_Header.Name[0] = '\0';
  m_Header.Size = 0;
  m_Header.StartOffset = 0;

  m_Header.Unknown[0] = 0;
  m_Header.Unknown[1] = 0;
  m_Header.Unknown[2] = 0;
  m_Header.Unknown[3] = 0;
  m_Header.Unknown[4] = 0;
  m_Header.Unknown[5] = 0;
  m_Header.Unknown[6] = 0;
  m_Header.Unknown[7] = 0;

  m_SkipData     = false;
  m_HeaderRead   = false;
  m_DataRead     = false;
  m_pData        = NULL;
 }
/*===========================================================================
 *		End of Class CMwEsmBaseRecord Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmBaseRecord Method - void Destroy (void);
 *
 * Deletes the contents of the record.
 *
 *=========================================================================*/
void CMwEsmBaseRecord::Destroy (void) {
  DEFINE_FUNCTION("CMwEsmBaseRecord::Destroy()");

  m_Header.Size    = 0;
  m_Header.Name[0] = '\0';
  m_HeaderRead     = false;
  m_DataRead       = false;

	/* Delete any allocated data */
  DestroyPointer(m_pData);
 }
/*===========================================================================
 *		End of Class Method CMwEsmBaseRecord::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmBaseRecord Method - CMwEsmBaseRecord* Create (void);
 *
 * Static method which creates and returns a new pointer of the class.
 *
 *=========================================================================*/
CMwEsmBaseRecord* CMwEsmBaseRecord::Create (void) {
  DEFINE_FUNCTION("CMwEsmBaseRecord::Create()");
  CMwEsmBaseRecord* pNewObject;

  CreatePointer(pNewObject, CMwEsmBaseRecord);
  return (pNewObject);
 }
/*===========================================================================
 *		End of Class Method CMwEsmBaseRecord::Create()
 *=========================================================================*/

/*===========================================================================
 *
 * Class CMwEsmBaseRecord Method - bool Read (File);
 *
 * Reads the record from the current position in the given file.  Returns
 * false on any error.
 *
 *=========================================================================*/
bool CMwEsmBaseRecord::Read (CGenFile& File) {
  bool Result;

	/* Input the record header information */
  if (!m_HeaderRead) 
    Result = ReadHeader(File);
  else
    Result = true;

	/* Only read the data if required */
  if (!m_SkipData && Result) 
    Result = ReadData(File);
  else if (m_SkipData)
    Result = SkipData(File);

	/* Check for overreading of data */
  if (File.Tell() != GetEndOffset()) {
    ErrorHandler.AddError(ERR_READFILE, "Overread of ESM record by %d bytes!", File.Tell() - GetEndOffset());
    return (false);
   }

  return (Result);
 }
/*===========================================================================
 *		End of Class Method CMwEsmBaseRecord::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmBaseRecord Method - bool ReadData (File);
 *
 * Protected clas method to input the data at the current position in
 * the file.  Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmBaseRecord::ReadData (CGenFile& File) {
  bool Result;

	/* Attempt to allocate the data buffer */
  ReplaceString(&m_pData, m_Header.Size);

	/* Attempt to input the data */
  Result = File.Read(m_pData, m_Header.Size);

  m_DataRead = Result;
  return (Result);  
 }
/*===========================================================================
 *		End of Class Method CMwEsmBaseRecord::ReadData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmBaseRecord Method - bool WriteData (File);
 *
 * Protected clas method to output the data to the current position in
 * the file.  Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmBaseRecord::WriteData (CGenFile& File) {
  bool Result;

	/* Attempt to output the data if required */
  if (m_pData != NULL && m_Header.Size > 0)
    Result = File.Write(m_pData, m_Header.Size);
  else
    Result = true;

  return (Result); 
 }
/*===========================================================================
 *		End of Class Method CMwEsmBaseRecord::WriteData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmBaseRecord Method - bool ReadHeader (File);
 *
 * Protected class method to input the record header from the current 
 * position in the given file.  Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmBaseRecord::ReadHeader (CGenFile& File) {
  bool Result;

	/* Attempt to input header from current position in file */
  Result = ReadRecordHeader(File, m_Header);

  m_HeaderRead = Result;
  return (Result);
 }
/*===========================================================================
 *		End of Class Method CMwEsmBaseRecord::ReadHeader()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmBaseRecord Method - bool ReadRecordHeader (File, Header);
 *
 * Static method to provide input of the base record header from the 
 * current position in the given file.  Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmBaseRecord::ReadRecordHeader (CGenFile& File, mwesmbaseheader_t& Header) {
  bool Result;

	/* Attempt to input header from current position in file */
  Result = File.Tell(Header.StartOffset);
  if (Result) Result = File.Read((char *) &Header, MWESM_BASEHEADER_SIZE);

  return (Result);
 }
/*===========================================================================
 *		End of Class Method bool CMwEsmBaseRecord::ReadRecordHeader()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmBaseRecord Method - bool SkipData (File);
 *
 * Protected class method. Sets the pointer to the end of the record data.
 * Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmBaseRecord::SkipData (CGenFile& File) {
  bool Result;

  Result = File.Seek(m_Header.StartOffset + m_Header.Size + MWESM_BASEHEADER_SIZE, SEEK_SET);
  return (Result);
 }
/*===========================================================================
 *		End of Class Method CMwEsmBaseRecord::SkipData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmBaseRecord Method - bool Write (File);
 *
 * Writes the record to the current position in the given file.  Returns
 * false on any error.
 *
 *=========================================================================*/
bool CMwEsmBaseRecord::Write (CGenFile& File) {
  bool Result;

	/* Output the header information if it exists */
  if (m_HeaderRead) 
    Result = WriteHeader(File);
  else
    Result = false;

	/* Output the data if it exists */
  if (m_DataRead && Result) 
    Result = WriteData(File);
  else
    Result = false;

  return (Result);
 }
/*===========================================================================
 *		End of Class Method CMwEsmBaseRecord::Write()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmBaseRecord Method - bool WriteHeader (File);
 *
 * Protected class method to output the record header to the current 
 * position in the given file.  Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmBaseRecord::WriteHeader (CGenFile& File) {
  bool Result;

	/* Attempt to output header to current position in file */
  Result = File.Write((char *) &m_Header, MWESM_BASEHEADER_SIZE);

  return (Result);
 }
/*===========================================================================
 *		End of Class Method CMwEsmBaseRecord::ReadHeader()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - bool MwEsmCompareHeaderName (pHeader1, cpHeader2);
 *
 * Global function to compare two ESM header names for equality. Returns
 * true if they are equal, ignoring case.
 *
 *=========================================================================*/
bool MwEsmCompareHeaderName (const char* pHeader1, const char* pHeader2) {
  int Result;

	/* Ensure valid input */
  assert(pHeader1 != NULL && pHeader2 != NULL);

	/* Compare strings without case sensitivity */
  Result = strnicmp(pHeader1, pHeader2, MWESM_HEADERNAME_SIZE);
  return (Result == 0);
 }
/*===========================================================================
 *		End of Function MwEsmCompareHeaderName()
 *=========================================================================*/
 

/*===========================================================================
 *
 * Function - bool MwEsmReadSubrecordName (pName, StringSize);
 *
 * Inputs a name/string size pair from a subrecord.  Returns false on any
 * error.  The input name must be at least 4 bytes in size.
 *
 *=========================================================================*/
bool MwEsmReadSubrecordName (CGenFile& File, char* pName, long& StringSize) {
  bool Result;

  Result = File.Read(pName, MWESM_HEADERNAME_SIZE);
  if (Result) Result = File.ReadLong(StringSize);
  
  return (Result);
 }
/*===========================================================================
 *		End of Function MwEsmReadSubrecordName()
 *=========================================================================*/
