/*===========================================================================
 *
 * File:	Mwesm_Lock.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Defines the CMwEsmRecordLock class.
 *
 *=========================================================================*/
#ifndef __MWESM_LOCK_H
#define __MWESM_LOCK_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "mwesm_baserec.h"
  #include "mwesm_string.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/


/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/
#pragma pack(push, 1)


	/* Repair data structure */
  typedef struct {
    float	Weight;
    long	Value;
    float	Quality;
    long 	Uses;
   } mwesmlockdata_t;
		
#pragma pack(pop)			
/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CMwEsmRecordLock Definition
 *
 * Defines the TES3 ESM base record class.
 *
	NAME = Item ID, required
	MODL = Model Name, required
	FNAM = Item Name, required
	LKDT = Lock Data (16 bytes), required
		
	ITEX = Inventory Icon
	SCRI = Script Name
 *
 *=========================================================================*/
class CMwEsmRecordLock : public CMwEsmBaseRecord {

  /*---------- Begin Private Class Members ----------------------*/
private:
  CMwEsmString		m_ID;
  CMwEsmString		m_Model;
  CMwEsmString		m_Name;
  CMwEsmString		m_Icon;
  CMwEsmString		m_Enchantment;
  CMwEsmString		m_Script;

  mwesmlockdata_t	m_Data;


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Input/output the weapon record */
  virtual bool ReadData    (CGenFile& File);
  virtual bool WriteData   (CGenFile& File);


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CMwEsmRecordLock();
  virtual ~CMwEsmRecordLock() { Destroy(); }
  virtual void Destroy (void);

  	/* Returns a new pointer of itself */
  static CMwEsmBaseRecord* Create (void);

	/* Get class members */
  const char*  GetTypeString    (void) const { return ("Lockpick"); }
  virtual bool IsBaseRecord	(void) const { return (false); }

  virtual bool OutputItem (CGenFile& File);

 };
/*===========================================================================
 *		End of Class CMwEsmRecordLock Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Mwesm_tes3.H
 *=========================================================================*/

