/*===========================================================================
 *
 * File:	Mwesm_Prob.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "Mwesm_Prob.h"
#include "string.h"
#include "assert.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("Mwesm_Prob.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmRecordProb Constructor
 *
 *=========================================================================*/
CMwEsmRecordProb::CMwEsmRecordProb () {
  
 }
/*===========================================================================
 *		End of Class CMwEsmRecordProb Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmRecordProb Method - void Destroy (void);
 *
 * Deletes the contents of the record.
 *
 *=========================================================================*/
void CMwEsmRecordProb::Destroy (void) {
  DEFINE_FUNCTION("CMwEsmRecordProb::Destroy()");

	/* Destroy all the strings */
  m_ID.Destroy();
  m_Model.Destroy();
  m_Name.Destroy();
  m_Icon.Destroy();
  m_Enchantment.Destroy();
  m_Script.Destroy();

	/* Call the base class record */
  CMwEsmBaseRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CMwEsmRecordProb::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmRecordProb Method - CMwEsmBaseRecord* Create (void);
 *
 * Static method which creates and returns a new pointer of the class.
 *
 *=========================================================================*/
CMwEsmBaseRecord* CMwEsmRecordProb::Create (void) {
  DEFINE_FUNCTION("CMwEsmRecordProb::Create()");
  CMwEsmRecordProb* pNewObject;

  CreatePointer(pNewObject, CMwEsmRecordProb);
  return (pNewObject);
 }
/*===========================================================================
 *		End of Class Method CMwEsmRecordProb::Create()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmRecordProb Method - bool OutputItem (File);
 *
 *=========================================================================*/
bool CMwEsmRecordProb::OutputItem (CGenFile& File) {
  bool Result;

  Result = File.Printf ("\"%s\", \"%s\", %s, %0.2f, %d\n", m_ID.GetString(), m_Name.GetString(), GetTypeString(), m_Data.Weight, (int)m_Data.Value);

  return (Result);
 }
/*===========================================================================
 *		End of Class Method CMwEsmRecordProb::OutputItem()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmRecordProb Method - bool ReadData (File);
 *
 * Protected clas method to input the data at the current position in
 * the file.  Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmRecordProb::ReadData (CGenFile& File) {
  DEFINE_FUNCTION("CMwEsmRecordProb::ReadData()");
  bool Result;
  long RecordSize;
  char NameBuffer[MWESM_HEADERNAME_SIZE];
  long EndOffset = GetEndOffset();

  //SystemLog.Printf ("Weapon Start 0x%08lX", File.Tell());

  while (File.Tell() < EndOffset) {

		/* Input the subrecord name and size */
    Result = MwEsmReadSubrecordName(File, NameBuffer, RecordSize);
    if (!Result) return (false);

		/* Determine which subrecord was input */
    if (MwEsmCompareHeaderName(NameBuffer, "NAME")) 
      Result = m_ID.ReadString(File, NameBuffer, RecordSize);
    else if (MwEsmCompareHeaderName(NameBuffer, "MODL")) 
      Result = m_Model.ReadString(File, NameBuffer, RecordSize);
    else if (MwEsmCompareHeaderName(NameBuffer, "FNAM")) 
      Result = m_Name.ReadString(File, NameBuffer, RecordSize);
    else if (MwEsmCompareHeaderName(NameBuffer, "ITEX")) 
      Result = m_Icon.ReadString(File, NameBuffer, RecordSize);
    else if (MwEsmCompareHeaderName(NameBuffer, "ENAM")) 
      Result = m_Enchantment.ReadString(File, NameBuffer, RecordSize);
    else if (MwEsmCompareHeaderName(NameBuffer, "SCRI")) 
      Result = m_Script.ReadString(File, NameBuffer, RecordSize);
    else if (MwEsmCompareHeaderName(NameBuffer, "PBDT")) {
      ASSERT(sizeof(mwesmprobdata_t) == RecordSize);
      Result = File.Read((char *)&m_Data, sizeof(mwesmprobdata_t));
     }
    else {	/* Unknown subrecord */
      MWESM_BADSUBRECORD(NameBuffer, File);
      return (false);
     }

   } /* End of while reading record */

  m_DataRead = true;
  return (true);
 }
/*===========================================================================
 *		End of Class Method CMwEsmRecordProb::ReadData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmRecordProb Method - bool WriteData (File);
 *
 * Protected class method to output the data to the current position in
 * the file.  Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmRecordProb::WriteData (CGenFile& File) {
  bool Result;

  Result = m_ID.Write(File);
  if (Result) Result = m_Model.Write(File);
  if (Result) Result = m_Name.Write(File);
  if (Result) Result = m_Icon.Write(File);
  if (Result) Result = m_Enchantment.Write(File);
  if (Result) Result = m_Script.Write(File);
  if (Result) Result = File.Write((char *)&m_Data, sizeof(mwesmprobdata_t));

  return (Result); 
 }
/*===========================================================================
 *		End of Class Method CMwEsmRecordProb::WriteData()
 *=========================================================================*/


