/*===========================================================================
 *
 * File:	Mwesm_String.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Wednesday, September 04, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "Mwesm_String.h"
#include "string.h"
#include "assert.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("Mwesm_String.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmString Constructor
 *
 *=========================================================================*/
CMwEsmString::CMwEsmString () {
  m_pString = NULL;
  m_StringSize = 0;
 }
/*===========================================================================
 *		End of Class CMwEsmString Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmString Method - void Destroy (void);
 *
 * Deletes the contents of the record.
 *
 *=========================================================================*/
void CMwEsmString::Destroy (void) {
  DEFINE_FUNCTION("CMwEsmString::Destroy()");

  DestroyArrayPointer(m_pString);
  m_StringSize = 0;
  m_IsLoaded = false;
 }
/*===========================================================================
 *		End of Class Method CMwEsmString::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmString Method - bool Read (File);
 *
 * Protected clas method to input the data at the current position in
 * the file.  Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmString::Read (CGenFile& File) {
  DEFINE_FUNCTION("CMwEsmString::Read()");
  bool Result;

	/* Delete the current data */
  Destroy();

  Result = File.Read(m_Name, MWESM_HEADERNAME_SIZE);
  if (Result) Result = File.ReadLong(m_StringSize);
  if (!Result) return (false);

	/* Allocate and read the string, size includes the nul character */
  if (m_StringSize > 0) {
    CreateArrayPointer(m_pString, char, m_StringSize);
    Result = File.Read(m_pString, m_StringSize);
    if (!Result) return (false);
   }

  m_IsLoaded = true;
  return (true);
 }
/*===========================================================================
 *		End of Class Method CMwEsmString::Read()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmString Method - bool ReadString (File, pName, StringSize);
 *
 * Inputs a string from the current position in the file using the input
 * name and stringsize.  Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmString::ReadString (CGenFile& File, const char* pName, const long StringSize) {
  DEFINE_FUNCTION("CMwEsmString::ReadString()");
  bool Result;

	/* Copy the name and string size */
  strncpy(m_Name, pName, MWESM_HEADERNAME_SIZE);
  m_StringSize = StringSize;

	/* Input the string data including the nul character */
  if (m_StringSize > 0) {
    CreateArrayPointer(m_pString, char, m_StringSize);
    Result = File.Read(m_pString, m_StringSize);
    if (!Result) return (false);
   }

  m_IsLoaded = true;
  return (true);
 }
/*===========================================================================
 *		End of Class Method CMwEsmString::ReadString()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CMwEsmString Method - bool Write (File);
 *
 * Protected class method to output the data to the current position in
 * the file.  Returns false on any error.
 *
 *=========================================================================*/
bool CMwEsmString::Write (CGenFile& File) {
  bool Result;

	/* Ignore if the string was not input */
  if (!IsLoaded()) return (true);

	/* Input the string name and size */
  Result = File.Write(m_Name, MWESM_HEADERNAME_SIZE);
  if (Result) Result = File.WriteLong(m_StringSize);
  if (!Result) return (false);

	/* Output the string, size includes the nul character */
  if (m_StringSize > 0) {
    Result = File.Write(m_pString, m_StringSize);
    if (!Result) return (false);
   }

  return (true); 
 }
/*===========================================================================
 *		End of Class Method CMwEsmString::Write()
 *=========================================================================*/


