/*===========================================================================
 *
 * File:	TamMapLocTypes.JAVA
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Monday, August 27, 2001
 *
 * Defines the class for holding various location type properties such
 * as colors, zoom levels, etc...
 *
 *=========================================================================*/

	/* Import Packages */
import java.awt.*;
import java.io.*;


/*===========================================================================
 *
 * Begin Class TamMapLocTypes
 *
 * Holds arrays of information related to the location types.
 *
 *=========================================================================*/
public class TamMapLocTypes {

  /*---------- Class Constants -----------------------------------------*/

	/* Defines the maximum size of the arrays */
  static final int MAX_LOC_TYPES = 32;

	/* Default color indices for objects */
  static final Color TAMMAP_COLORS[] = {	
	new Color(0,0,0),		// 0  - Black 
	new Color(0,0,128),		// 1  - Water
	new Color(64,128,64),		// 2  - Province 1
	new Color(128,64,128),		// 3  - Province 2
	new Color(64,64,128),		// 4  - Province 3
	new Color(128,64,64),		// 5  - Province 4
	new Color(128,128,128),		// 6  - Province 5
	new Color(128,0,0),		// 7  - Province 6
	new Color(0,128,0),		// 8  - Province 7
	new Color(128,128,64),		// 9  - Province 8
	new Color(64,128,128),		// 10 - Province 9
	new Color(80,80,200),		// 11 - Province 10
	new Color(255,255,0),		// 12 - Border 1
	new Color(228,228,0),		// 13 - Border 2
	new Color(172,172,0),		// 14 - Border 3
	new Color(64,64,255),		// 15 - River
	new Color(172,64,172),		// 16 - Road 1
	new Color(150,64,150),		// 17 - Road 2
	new Color(128,32,128),		// 18 - Road 3
	new Color(0,255,255),		// 19 - City 1
	new Color(0,172,172),		// 20 - City 2
	new Color(0,128,128),		// 21 - City 3
	new Color(255,172,0),		// 22 - Ruin 1
	new Color(172,128,0),		// 23 - Ruin 2
	new Color(128,64,0),		// 24 - Ruin 3
	new Color(172,0,172),		// 25 - Misc 1
	new Color(128,0,128),		// 26 - Misc 2
	new Color(64,0,64),		// 27 - Misc 3
	new Color(128,128,255),		// 28 - Trade 1
	new Color(32,32,255),		// 29 - Trade 2
	new Color(200,200,200),		// 30 - Free 1
	new Color(172,172,172),		// 31 - Free 2
	new Color(255,255,255) };	// 32 - White

	/* Default color used in various places */
  static final Color DEFAULT_COLOR = new Color(255,255,255);
					
  /*---------- End of Class Constants --------------------------------*/


/*===========================================================================
 *
 * Method - Color GetColor (LocType);
 *
 * Returns the color associated with the given location type.  Returns a
 * default color if the input type is not valid.
 *
 *=========================================================================*/
static public Color GetColor (int LocType) {
	
	/* Ensure a valid location type */
  if (LocType < 0 || LocType >= MAX_LOC_TYPES) return (DEFAULT_COLOR);

  return (TAMMAP_COLORS[LocType]);
 }
/*===========================================================================
 *		End of Method GetColor()
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Default Color Methods
 *
 *=========================================================================*/
static public Color GetWaterColor   () { return (TAMMAP_COLORS[1]); }
static public Color GetDefaultColor () { return (DEFAULT_COLOR); }
/*===========================================================================
 *		End of Default Color Methods
 *=========================================================================*/




 }
/*===========================================================================
 *		End of Class TamMapLocTypes
 *=========================================================================*/


/*===========================================================================
 *		End of File TamMapLocTypes.JAVA
 *=========================================================================*/
