/*===========================================================================
 *
 * File:	Gengraphaxis.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Thursday, October 17, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "gengaxis.h"
#include "dl_math.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("GenGAxis.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/



/*===========================================================================
 *
 * Class CGenGraphAxis Constructor
 *
 *=========================================================================*/
CGenGraphAxis::CGenGraphAxis () {
  DEFINE_FUNCTION("CGenGraphAxis::CGenGraphAxis()");
  m_StartCoor = 0;
  m_EndCoor   = 10;

  m_FastCtoLFactor = 1;
  m_FastCtoLBase   = 0;

  m_pLabel         = NULL;
  m_DisplayLabel   = true;
  m_DisplayTicks   = true;
  m_XLabelOffset   = 0;
  m_YLabelOffset   = 0;
  m_InvertAxis     = false;
  m_TickSpacing    = 2.0f;
  m_TickLength     = 4;
  m_NumAutoTicks   = 5;
  m_AutoTick       = true;

  m_BGColor    = GENGRAPH_COLOR_GRAY;
  m_LabelColor = GENGRAPH_COLOR_BLACK;
  m_pBGBrush = new CBrush(m_BGColor);
  m_pAxisPen = new CPen(PS_SOLID, 1, GENGRAPH_COLOR_BLACK);
  m_pLabelFont = new CFont();
  m_pTickFont = new CFont();
  ValidateNewPointer(m_pLabelFont);
  ValidateNewPointer(m_pTickFont);
  ValidateNewPointer(m_pBGBrush);
  ValidateNewPointer(m_pAxisPen);

  m_pLabelFont->CreatePointFont(90, _T("MS Sans Serif"));
  m_pTickFont->CreatePointFont(75, _T("Arial"));
  SetFormat(_T("%g"));

  ComputeCoorTransform();
  AutoComputeTicks();
 }
/*===========================================================================
 *		End of Class CGenGraphAxis Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraphAxis Destructor
 *
 *=========================================================================*/
CGenGraphAxis::~CGenGraphAxis () {
  DEFINE_FUNCTION("CGenGraphAxis::~CGenGraphAxis()");

  Destroy();

  DestroyPointer(m_pLabel);
  DestroyPointer(m_pAxisPen);
  DestroyPointer(m_pBGBrush);
  DestroyPointer(m_pLabelFont);
  DestroyPointer(m_pTickFont);
 }
/*===========================================================================
 *		End of Class CGenGraphAxis Destructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraphAxis Method - void AutoComputeTicks (void);
 *
 *=========================================================================*/
void CGenGraphAxis::AutoComputeTicks (void) {
  //DEFINE_FUNCTION("CGenGraphAxis::AutoComputeTicks()");
  if (!m_AutoTick) return;
  m_TickSpacing = (float) GetNiceTickLengthC(m_StartCoor, m_EndCoor, m_NumAutoTicks);
 }
/*===========================================================================
 *		End of Class Method CGenGraphAxis::AutoComputeTicks()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraphAxis Method - void ComputeCoorTransform (void);
 *
 *=========================================================================*/
void CGenGraphAxis::ComputeCoorTransform (void) {

	/* Ensure a valid range */
  if (m_EndCoor == m_StartCoor) {
    m_FastCtoLFactor = 1.0;
    m_FastCtoLBase   = 0.0;
    return;
   }

	/* Different cases for a X/Y axis */
  if (IsYAxis()) 
    m_FastCtoLFactor = -(GetAxisLength() - 1) / (m_EndCoor - m_StartCoor);
  else
    m_FastCtoLFactor = (GetAxisLength() - 1) / (m_EndCoor - m_StartCoor);

  m_FastCtoLBase = (GetAxisOrigin() - m_FastCtoLFactor*m_StartCoor);
 }
/*===========================================================================
 *		End of Class Method CGenGraphAxis::ComputeCoorTransform()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraphAxis Method - int DrawBG (pDC);
 *
 *=========================================================================*/
int CGenGraphAxis::DrawBG (CDC* pDC) {
  //DEFINE_FUNCTION("CGenGraphAxis::DrawBG()");

#if defined(__BORLANDC__)
  pDC->FillRect(GetLeft(), GetTop(), GetWidth(), GetHeight(), *m_pBGBrush);
#else
  pDC->FillSolidRect(GetLeft(), GetTop(), GetWidth(), GetHeight(), m_BGColor);
#endif

  return (0);
 }
/*===========================================================================
 *		End of Class Method CGenGraphAxis::DrawBG()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraphAxis Event - int OnPaint (pDC);
 *
 *=========================================================================*/
int CGenGraphAxis::OnPaint (CDC* pDC) {

  if (IsVisible()) {
    DrawBG(pDC);
    DrawAxis(pDC);
    DrawLabel(pDC);
    DrawTicks(pDC);
   }

  return (0);
 }
/*===========================================================================
 *		End of Class Event CGenGraphAxis::OnPaint()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenGraphAxis Method - void SetCoordinates (Start, End);
 *
 *=========================================================================*/
void CGenGraphAxis::SetCoordinates (const graphdata_t Start, const graphdata_t End) {
  m_StartCoor = Start;
  m_EndCoor   = End;
  ComputeCoorTransform();
  AutoComputeTicks();
 }

void CGenGraphAxis::SetEndCoor (const graphdata_t End) {
  m_EndCoor   = End;
  ComputeCoorTransform();
  AutoComputeTicks();
 }

void CGenGraphAxis::SetStartCoor (const graphdata_t Start) {
  m_StartCoor   = Start;
  ComputeCoorTransform();
  AutoComputeTicks();
 }

void CGenGraphAxis::SetLabelFont (const TCHAR* pFontname, const int PointSize) {
  m_pLabelFont->DeleteObject();
  m_pLabelFont->CreatePointFont(PointSize, pFontname);
 }

void CGenGraphAxis::SetTickFont (const TCHAR* pFontname, const int PointSize) {
  m_pTickFont->DeleteObject();
  m_pTickFont->CreatePointFont(PointSize, pFontname);
 }
/*===========================================================================
 *		End of Class Method CGenGraphAxis::SetCoordinates()
 *=========================================================================*/

