/*===========================================================================
 *
 * File:	GenGraphAxis.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Thursday, October 17, 2002
 *
 * Description
 *
 *=========================================================================*/
#ifndef __GENGRAPHAXIS_H
#define __GENGRAPHAXIS_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "dl_str.h"
  #include "GenGBase.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/
	
  #define GGAXIS_TICKFORMAT_SIZE 15
/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CGenGraphAxis Definition
 *
 * Description
 *
 *=========================================================================*/
class CGenGraphAxis : public CGenGraphBase {

  /*---------- Begin Private Class Members ----------------------*/
protected:
  TCHAR*	m_pLabel;
  bool		m_DisplayLabel;
  bool		m_InvertAxis;
  int		m_XLabelOffset;
  int		m_YLabelOffset;

  graphdata_t	m_TickSpacing;
  int		m_TickLength;
  bool		m_DisplayTicks;
  bool		m_AutoTick;
  int		m_NumAutoTicks;
  TCHAR		m_NumFormat[GGAXIS_TICKFORMAT_SIZE+1];

  graphdata_t	m_StartCoor;
  graphdata_t	m_EndCoor;

  graphdata_t	m_FastCtoLFactor;
  graphdata_t	m_FastCtoLBase;

  COLORREF	m_BGColor;
  COLORREF	m_LabelColor;
  COLORREF	m_TickColor;
  CBrush*	m_pBGBrush;
  CPen*		m_pAxisPen;
  CFont*	m_pLabelFont;
  CFont*	m_pTickFont;


  /*---------- Begin Protected Class Methods --------------------*/
protected:

	/* Helper draw functions */
  virtual int DrawAxis  (CDC* pDC) = 0;
  virtual int DrawBG    (CDC* pDC);
  virtual int DrawLabel (CDC* pDC) = 0;
  virtual int DrawTicks (CDC* pDC) = 0;


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CGenGraphAxis();
  virtual ~CGenGraphAxis();
  //virtual void Destroy (void);

	/* Compute a nice tick length */
  void AutoComputeTicks (void);

    	/* Coordinate tranformations */
  void		ComputeCoorTransform (void);
  int		FastCoorToLogical    (const graphdata_t Value) const;
  graphdata_t	FastLogicalToCoor    (const int Value) const;
  virtual int	LimitCoorToLogical   (const graphdata_t Value) const;

	/* Get class members */
  graphdata_t  GetEndCoor       (void) const { return (m_EndCoor); }
  graphdata_t  GetStartCoor     (void) const { return (m_StartCoor); }
  const TCHAR* GetLabel		(void) const { return (m_pLabel); }
          bool IsInverted	(void) const { return (m_InvertAxis); }
  virtual int  GetAxisLength	(void) const { return (1); }
  virtual int  GetAxisOrigin	(void) const { return (1); }

  virtual bool IsXAxis		(void) const { return (false); }
  virtual bool IsYAxis		(void) const { return (false); }

	/* Is the given value visible on the axis? */
  bool IsValidPoint (const graphdata_t Data) const { return (Data >= m_StartCoor && Data <= m_EndCoor); }

  	/* Events */
  virtual int OnPaint (CDC* pDC);

	/* Set class members */
	  void SetLabel	      (const TCHAR* pString) { ReplaceString(&m_pLabel, pString); }
	  void SetLabelColor  (const COLORREF Color) { m_LabelColor = Color; }
	  void SetLabelOffset (const int X, const int Y) { m_XLabelOffset = X; m_YLabelOffset = Y; }
	  void SetDrawLabel   (const bool Flag)      { m_DisplayLabel = Flag; }
	  void SetLabelFont   (const TCHAR* pFontname, const int PointSize);
	  void SetTickColor   (const COLORREF Color) { m_TickColor = Color; }
	  void SetDrawTicks   (const bool Flag)      { m_DisplayTicks = Flag; }
	  void SetTickFont    (const TCHAR* pFontname, const int PointSize);
	  void SetTickSpacing (const graphdata_t Value) { m_TickSpacing = Value; }
	  void SetTickLength  (const int Value)         { m_TickLength  = Value; }
	  void SetInvertAxis  (const bool Flag)         { m_InvertAxis = Flag; }
	  void SetNumAutoTicks(const int Value)         { m_NumAutoTicks = Value; AutoComputeTicks(); }
  virtual void SetCoordinates (const graphdata_t Start, const graphdata_t End);
	  void SetEndCoor     (const graphdata_t End);
	  void SetStartCoor   (const graphdata_t Start);
	  void SetFormat      (const TCHAR* pFormat) { strnncpy(m_NumFormat, pFormat, GGAXIS_TICKFORMAT_SIZE); }

 };
/*===========================================================================
 *		End of Class CGenGraphAxis Definition
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Inline Methods
 *
 *=========================================================================*/

	/* Convert an axis coordinate to a logical coordinate */
  inline int CGenGraphAxis::FastCoorToLogical (const graphdata_t Value) const {

    return (int) (Value*m_FastCtoLFactor + m_FastCtoLBase);
   }

	/* Same as FastCoorToLogical() but will clip the given point to
	   fit into the current Axis range */
  inline int CGenGraphAxis::LimitCoorToLogical (const graphdata_t Value) const {

    if (Value <= m_StartCoor) 
      return (FastCoorToLogical(m_StartCoor));
      //return GetAxisOrigin() + 1;
    else if (Value >= m_EndCoor) 
      return (FastCoorToLogical(m_EndCoor));
      //return GetAxisOrigin() + GetAxisLength() - 2;
    else
      return (FastCoorToLogical(Value));
   }

	/* Converts a logical (pixel) value to an axis coordinate value */
  inline graphdata_t CGenGraphAxis::FastLogicalToCoor (const int Value) const {
    if (m_FastCtoLFactor != 0) 
      return (Value - m_FastCtoLBase) / m_FastCtoLFactor;
    else
      return (Value - m_FastCtoLBase);
   }

/*===========================================================================
 *		End of Inline Methods
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File GenGraphAxis.H
 *=========================================================================*/
