/*===========================================================================
 *
 * File:	Genyaxis.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Thursday, October 17, 2002
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "stdafx.h"
#include "genyaxis.h"
#include "dl_math.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("GenYAxis.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenYAxis Method - int DrawAxis (CDC* pDC);
 *
 *=========================================================================*/
int CGenYAxis::DrawAxis (CDC* pDC) {
  //DEFINE_FUNCTION("CGenYAxis::DrawAxis()");

  pDC->SelectObject(m_pAxisPen);

  pDC->MoveTo (GetRight(), GetTop());
  pDC->LineTo (GetRight(), GetBottom()+1);

  return (0);
 }
/*===========================================================================
 *		End of Class Method CGenYAxis::DrawAxis()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenYAxis Method - int DrawLabel (pDC);
 *
 *=========================================================================*/
int CGenYAxis::DrawLabel (CDC* pDC) {
  CSize Extents;
  int   X;
  int   Y;

	/* Ignore if nothing to display */
  if (!m_DisplayLabel || m_pLabel == NULL) return (0);

  pDC->SelectObject(m_pLabelFont);
  Extents = pDC->GetTextExtent(m_pLabel);

  Y = GetTop() - Extents.cy - 1;

  if (m_InvertAxis)
    X = GetLeft() - Extents.cx;
  else
    X = GetRight();
  
  pDC->SetTextColor(m_LabelColor);
  pDC->TextOut(X + m_XLabelOffset, Y + m_YLabelOffset, m_pLabel);

  return (0);
 }
/*===========================================================================
 *		End of Class Method CGenYAxis::DrawLabel()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CGenYAxis Method - int DrawTicks (pDC);
 *
 *=========================================================================*/
int CGenYAxis::DrawTicks (CDC* pDC) {
  float NewAxisStart;
  float NewAxisEnd;
  float GraphY;
  CSize Extents;
  int   X;
  int   Y;
  int   LabelHeight;
  int   LastY;

  TCHAR Buffer[64];

	/* Ignore if ticks are not to be displayed */
  if (!m_DisplayTicks || m_TickSpacing <= 0) return (0);

	/* Compute nice axis start/end */
  NewAxisStart = (float) floor(m_StartCoor/m_TickSpacing)*m_TickSpacing;
  NewAxisEnd   = (float) ceil(m_EndCoor/m_TickSpacing)*m_TickSpacing;
  pDC->SelectObject(m_pTickFont);
  pDC->SelectObject(m_pAxisPen);
  pDC->SelectObject(m_pBGBrush);
  Extents = pDC->GetTextExtent(_T("0"));
  LabelHeight = Extents.cy + 1;
  GraphY = NewAxisStart;
  LastY  = -1;
  X = m_InvertAxis ? GetLeft() : GetRight();

	/* Output each tick mark */
  while (GraphY <= NewAxisEnd + 0.5*m_TickSpacing) {

		/* Ensure that the first and last ticks are output */
    if (GraphY < m_StartCoor) {
      Y = FastCoorToLogical(m_StartCoor);
      sprintf (Buffer, m_NumFormat, m_StartCoor);
     }
    else if (GraphY > m_EndCoor) {
      Y = FastCoorToLogical(m_EndCoor);
      sprintf (Buffer, m_NumFormat, m_EndCoor);
     }
    else {
      Y = FastCoorToLogical(GraphY);
      sprintf (Buffer, m_NumFormat, GraphY);
     }

     		/* Ignore if last tick mark was too close */
    if (LastY > 0 && abs(Y - LastY) < LabelHeight ) {
      GraphY = GraphY + m_TickSpacing;
      continue;
     }

		/* Output the tick mark */
    pDC->MoveTo(X, Y);
    LastY = Y;

    if (m_InvertAxis)
       pDC->LineTo(X + m_TickLength, Y);
    else
       pDC->LineTo(X - m_TickLength, Y);

		/* Output the tick label */
    Extents = pDC->GetTextExtent(Buffer);

    if (m_InvertAxis)
      pDC->TextOut(X + m_TickLength, Y - (Extents.cy/2), Buffer);
    else
      pDC->TextOut(X - m_TickLength - Extents.cx, Y - (Extents.cy/2), Buffer);

    GraphY = GraphY + m_TickSpacing;
   } 

  return (0);
 }
/*===========================================================================
 *		End of Class Method CGenYAxis::DrawTicks()
 *=========================================================================*/


