/*===========================================================================
 *
 * File:	Tammap.JAVA
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	Monday, August 27, 2001
 *
 * Description
 *
 *=========================================================================*/

	/* Import Packages */
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import TamMapLocTypes;
import TamMapRecord;
import TamMapOptions;
import TamMapArray;


/*===========================================================================
 *
 * Begin Class TamMap
 *
 * Main class for handling the Tamriel map parsing and output.
 *
 *=========================================================================*/
public class TamMap extends java.applet.Applet {

  /*---------- Begin Class Data Constants --------------------------*/
  /*---------- End of Class Data Constants -------------------------*/

  
  /*---------- Begin Class Data Members --------------------------*/

  Image		MapImage;		/* Stores the image of the map */
  Graphics      MapGraphics;		/* The graphics handle for image */

  TamMapArray   m_ProvinceArray;	/* Array of province map records */
  TamMapArray   m_LocationArray;	/* Array of general location map records */

  TamMapOptions m_MapOptions;		/* Holds options related to map input/output */

  /*---------- End of Class Data Members -------------------------*/


/*===========================================================================
 *
 * Method - int CheckParameters ();
 *
 * Checks and parses any input parameters for the applet.  Returns -1 on any
 * error.
 *
 *=========================================================================*/
int CheckParameters () {
  String ParamString;
  int    Value;
  
	/* Check for the pixel width of image */
  ParamString = getParameter("PixelWidth");

  if (ParamString != null) {
    Value = Integer.valueOf(ParamString).intValue();
    if (Value < 10 || Value > 1024) return (-1);
    m_MapOptions.m_PixelWidth = Value;
   }

	/* Check for the pixel height of image */
  ParamString = getParameter("PixelHeight");

  if (ParamString != null) {
    Value = Integer.valueOf(ParamString).intValue();
    if (Value < 10 || Value > 1024) return (-1);
    m_MapOptions.m_PixelHeight = Value;
   }

	/* Check for the search string */
  ParamString = getParameter("SearchString");

  if (ParamString != null) {
    m_MapOptions.SetSearchString(ParamString);
   }

  return (0);
 }
/*===========================================================================
 *		End of Function CheckParameters()
 *=========================================================================*/


/*===========================================================================
 *
 * Method - void ClearMapImage ();
 *
 * Clears the entire map image buffer.
 *
 *=========================================================================*/
public void ClearMapImage () {
  MapGraphics.setColor(TamMapLocTypes.GetWaterColor());
  MapGraphics.fillRect(0, 0, m_MapOptions.m_PixelWidth, m_MapOptions.m_PixelHeight);
 }
/*===========================================================================
 *		End of Function ClearMapImage()
 *=========================================================================*/


/*===========================================================================
 *
 * Method - public int CreateMapImage ();
 *
 * Attempts to create a new map image based on the current option and
 * map database information.
 *
 *=========================================================================*/
public int CreateMapImage () {

	/* Update the conversion factors */
  m_MapOptions.ComputeConversion();

	/* Delete the old map image, if any */
  ClearMapImage();

	/* Output province/main locations */
  m_ProvinceArray.CreateImage(MapGraphics, m_MapOptions);
  m_LocationArray.CreateImage(MapGraphics, m_MapOptions);

  return (0);
 }
/*===========================================================================
 *		End of Method int CreateMapImage()
 *=========================================================================*/


/*===========================================================================
 *
 * Class TamMap Method - public void init ();
 *
 *=========================================================================*/
public void init() {
  int Result;

	/* Create the map database arrays */
  m_ProvinceArray = new TamMapArray();
  m_LocationArray = new TamMapArray();

	/* Create the map options */
  m_MapOptions = new TamMapOptions();

	/* Check and parse any input parameters */
  Result = CheckParameters();

	/* Create the buffered map image */
  MapImage = createImage(m_MapOptions.m_PixelWidth, m_MapOptions.m_PixelHeight);
  MapGraphics = MapImage.getGraphics();
  
	/* Read and parse the map databases */
  //Result = m_ProvinceArray.LoadDatabase("http://shadow/TamMap/prov1.gdb", true);
  //Result = m_ProvinceArray.LoadDatabase("http://12.145.63.45/~uesp/tamriel/geography/prov1.gdb", true);
  Result = m_ProvinceArray.LoadDatabase(getCodeBase() + "prov1.gdb", true);
  //if (Result >= 0) Result = m_LocationArray.LoadDatabase("http://shadow/TamMap/main1.gdb", false);
  //if (Result >= 0) Result = m_LocationArray.LoadDatabase("http://12.145.63.45/~uesp/tamriel/geography/main1.gdb", false);
  if (Result >= 0) Result = m_LocationArray.LoadDatabase(getCodeBase() + "main1.gdb", false);
  //if (Result <  0) OutputError("Failed to load map database file!");

	/* Attempt to output database to image buffer */
  CreateMapImage();

  //m_ProvinceArray.DebugOutput();
  //m_LocationArray.DebugOutput();
 }
/*===========================================================================
 *		End of Class TamMap Method init()
 *=========================================================================*/


/*===========================================================================
 *
 * Class TamMap Method - void OutputError (Message);
 *
 * Displays the given error message to the Java window and stops the
 * applet.
 *
 *=========================================================================*/
public void OutputError (String Message) {

	/* Clear the current map image */
  ClearMapImage();

	/* Output the error message */
  MapGraphics.setColor(Color.red);
  MapGraphics.drawString("An error occurred in the Java applet!", 10, 10);
  MapGraphics.drawString(Message, 10, 30);

  repaint();
  System.exit(1);
 }
/*===========================================================================
 *		End of Class TamMap Method OutputError()
 *=========================================================================*/


/*===========================================================================
 *
 * Class TamMap Method - paint (graph);
 *
 *=========================================================================*/
public void paint (Graphics graph) {
  graph.drawImage(MapImage, 0, 0, this);
 }
/*===========================================================================
 *		End of Method paint()
 *=========================================================================*/


/*===========================================================================
 *
 * Method - void Pan (DX, DY);
 *
 * Attempts to pan the current view point of the map.  
 *
 *=========================================================================*/
public void Pan (float DX, float DY) {
  int Result;
  
	/* Attempt to zoom in/out */
  Result = m_MapOptions.Pan(DX, DY);
  if (Result < 0) return;

	/* Update the display */
  CreateMapImage();
  repaint();
 }
/*===========================================================================
 *		End of Method Pan()
 *=========================================================================*/


/*===========================================================================
 * 
 * Method - void Search (SearchString);
 *
 * Searches the map database for matching locations, outputting the results
 * to a new map image.
 *
 *=========================================================================*/
public void Search (String SearchString) {

	/* Update the map options */
  m_MapOptions.SetSearchString(SearchString);

	/* Update the map image */
  m_MapOptions.ZoomFullMap();
  CreateMapImage();
  repaint();
 }
/*===========================================================================
 *		End of Method Search()
 *=========================================================================*/


/*===========================================================================
 *
 * Class TamMap Method - public void start ();
 *
 *=========================================================================*/
public void start() {
 }
/*===========================================================================
 *		End of Class TamMap Method start()
 *=========================================================================*/


/*===========================================================================
 *
 * Method - void Zoom (Amount);
 *
 * Attempts to zoom into/out of the current view point of the map.  Amount
 * is a number representing the amount of zoom.  
 *	0.5 = Zoom in to half size
 *	1.0 = No change
 *	2.0 = Zoom out to double size
 *
 *=========================================================================*/
public void Zoom (float Amount) {
  int Result;
  	
	/* Attempt to zoom in/out */
  Result = m_MapOptions.Zoom(Amount);
  if (Result < 0) return;

	/* Update the display */
  CreateMapImage();
  repaint();
 }
/*===========================================================================
 *		End of Method Zoom()
 *=========================================================================*/


/*===========================================================================
 *
 * Method - void ZoomFullMap ();
 *
 * Resets the zoom level to show the entire map area.  
 *
 *=========================================================================*/
public void ZoomFullMap () {
  int Result;
  	
	/* Attempt to zoom in/out */
  Result = m_MapOptions.ZoomFullMap();
  if (Result < 0) return;

	/* Update the display */
  CreateMapImage();
  repaint();
 }
/*===========================================================================
 *		End of Method ZoomFullMap()
 *=========================================================================*/

 }
/*===========================================================================
 *		End of Class TamMap
 *=========================================================================*/


/*===========================================================================
 *		End of File Tammap.JAVA
 *=========================================================================*/
